# -*- coding: utf-8 -*-

"""Asyncore Service and connection implementation.

Implement a class for AsyncoreService and a NodeConnection with handlers.
"""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import socket
import threading
import asyncore


class RawHandler(object):  # pylint:disable=too-few-public-methods
    """Raw data handler."""
    def __init__(self, handler):
        self.handler = handler

    def __call__(self, data):
        """Call handler on raw data."""
        self.handler(data)


class NodeConnection(asyncore.dispatcher_with_send):
    """Handle the asyncore connection to one node."""

    RECV_LEN = 8192

    def __init__(self, archi, num,  # pylint:disable=too-many-arguments
                 event_handler, data_handler=None, service=None):
        asyncoremap = getattr(service, 'map', None)
        asyncore.dispatcher_with_send.__init__(self, map=asyncoremap)

        self.address = self._address(archi, num)

        self.event_handler = event_handler
        self.data_handler = data_handler

    def _address(self, archi, num):  # pylint:disable=no-self-use
        """Return socket address for archi/num."""
        return archi, int(num)

    def handle_data(self, data):
        """Call given data_handler."""
        # save value for thread safety, data_handler can change
        handler = self.data_handler
        if handler:
            handler(data)

    def start(self):
        """Connects to node serial port:"""

        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.connect(self.address)
        except:  # pylint:disable=broad-except,bare-except
            self.handle_error()

    def handle_connect(self):
        """Node connected."""
        self.event_handler('connect')

    def handle_close(self):
        """Close the connection and clear buffer."""
        self.close()
        self.event_handler('close')

    def close(self):
        """Safe close."""
        try:
            asyncore.dispatcher_with_send.close(self)
        except AttributeError:
            # When close is done on socket == None
            pass

    def handle_read(self):
        """Read bytes and run data handler."""
        data = self.recv(self.RECV_LEN)
        self.handle_data(data)

    def handle_error(self):
        """Error."""
        self.event_handler('error')


class AsyncoreService(object):
    """Asyncore wrapping in a Thread."""
    ASYNCORE_LOOP_KWARGS = {'timeout': 1, 'use_poll': True}

    def __init__(self):
        self.keep_alive = None
        self.map = {}
        self.thread = threading.Thread(target=self._loop)
        self.thread.deamon = True

    def start(self):
        """Start service.

        Register a 'keep_alive' dispatcher to maintain asyncore loop.
        Start asyncore loop in a thread.
        """
        self.keep_alive = self._keep_alive_dispatcher(self.map)
        self.thread.start()

    def _loop(self):
        """Run asyncore loop."""
        args = {'map': self.map}
        args.update(self.ASYNCORE_LOOP_KWARGS)
        return asyncore.loop(**args)

    def stop(self):
        """Stop service.
        All nodes connection should be closed before.

        Close the keep_alive and wait for thread to return.
        """
        self.keep_alive.close()
        self.thread.join()

    @staticmethod
    def _keep_alive_dispatcher(asyncoremap=None):
        """Stub tcp dispatcher to keep asyncore looping."""
        dispatcher = asyncore.dispatcher(map=asyncoremap)
        dispatcher.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        dispatcher.set_reuse_addr()
        dispatcher.bind(('', 0))
        dispatcher.listen(0)
        return dispatcher



# -*- coding: utf-8 -*-

"""Provide access to IoT-LAB experiments as MQTT agents"""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

__version__ = '0.6.1'



# -*- coding: utf-8 -*-

"""IoT-LAB MQTT client common"""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import os
import re
import cmd
import sys
import uuid
import readline
import functools


def clientid(name=None):
    """Return clientid for ``name``.

    If ``name`` is None, use hostname.
    """
    name = name or os.uname()[1]
    return '%s-%s' % (name, uuid.uuid4())


class CmdShell(cmd.Cmd, object):
    """Cmd shell class that handles string as utf8 strings."""

    def __init__(self):
        cmd.Cmd.__init__(self)
        self.input_encoding = sys.stdin.encoding

    def onecmd(self, line):
        """onecmd command wrapped to nicely handle lines and ValueError.

        * decode lines as using stdin encoding
        * replace nbsp in lines
        * Print help on ValueError
        * Print error on RuntimeError
        """
        try:
            line = self.decode_line(line, encoding=self.input_encoding)
            line = self.replace_nbsp(line)
            return cmd.Cmd.onecmd(self, line)

        except UnicodeDecodeError:
            print('Error while decoding input as utf-8')
            return 0

        except ValueError:
            # Error from do_command while handling `arg`
            command = self.parseline(line)[0]
            print('Error: Invalid arguments')
            self._print_usage(command)

        except RuntimeError as err:
            # Error from command execution, ex requests
            print('%s' % err)

    def _print_usage(self, command):
        """Print usage for command."""
        help_func = getattr(self, 'help_' + command)
        print('Usage: ', end='')
        help_func()

    @staticmethod
    def decode_line(line, encoding='utf-8'):
        """Decode ``line`` if is type ``bytes``.

        Allows managing arguments for both python2 and python3.
        cmd uses bytes in python2 and unicode string in python3
        """
        if isinstance(line, bytes):
            line = line.decode(encoding)
        return line

    @staticmethod
    def replace_nbsp(line):
        """Replace non breakable spaces by spaces."""
        return line.replace('\u00A0', ' ')

    @staticmethod
    def cmd_split(arg, maxsplit=0):
        """Split given command arg on whitespaces.

        Cannot use shlex.split as it has issues with unicode strings.
        """
        splitted = re.split(r'\s+', arg, maxsplit=maxsplit)
        # Remove empty values
        splitted = [a for a in splitted if a]
        return splitted

    def async_print_handle_readlinebuff(self):
        """Manages a clean readline buffer for asynchronous print.

        If there is content in readline buffer, add a newline before calling
        function and restore readline displayed string after function.
        """
        def _wrap(func):
            @functools.wraps(func)
            def _wrapped(*args, **kwargs):

                if readline.get_line_buffer():
                    sys.stdout.write('\n')

                ret = func(*args, **kwargs)

                sys.stdout.write(self.prompt)
                sys.stdout.write(readline.get_line_buffer())
                sys.stdout.flush()
                return ret

            return _wrapped
        return _wrap


def parser_add_site_arg(parser, group_help='Server agent IoT-LAB site name'):
    """Add server agent IoT-LAB site name argument."""
    group = parser.add_argument_group(group_help)
    group.add_argument('--site', help='Server agent site', required=True)


def parser_add_site_or_agenttopic_arg(
        parser, title='Server agent topic configuration',
        group_help='Server agent IoT-LAB site name or agent topic'):
    # pylint:disable=invalid-name
    """Add server agent IoT-LAB site name or agenttopic argument."""
    group = parser.add_argument_group(title, group_help)
    group.add_argument('--site', help='Server agent site')
    group.add_argument('--agenttopic', help='Agent topic overwrite')


def parser_verify_site_and_agenttopic(parser, opts):
    # pylint:disable=invalid-name
    """Verify that at least site or agenttopic is given."""
    if opts.site is None and opts.agenttopic is None:
        parser.error('at least one of --site and --agenttopic is required')

    if opts.agenttopic is not None:
        if opts.site is None and '{site}' in opts.agenttopic:
            parser.error('agenttopic contains "{site}" so --site is required')



# -*- coding:utf-8 -*-

"""MQTTCommon module tests."""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import time
import threading
import packaging.version
from iotlabmqtt import mqttcommon


class MQTTMessage(mqttcommon.mqtt.MQTTMessage):  # pylint:disable=R0903
    """Add 'eq' to MQTTMessage for Mock use."""

    def __eq__(self, other):
        # Compare internal dicts, except 'info' as not important for tests
        self_d = self._simple_dict(self)
        other_d = self._simple_dict(other)

        ret = self_d == other_d

        if not ret:
            print(self_d)
            print(other_d)

        return ret

    @staticmethod
    def _simple_dict(obj):
        """__dictw_ without values that should not compared."""
        obj_d = obj.__dict__.copy()
        obj_d.pop('info')
        obj_d.pop('reply_publisher', None)
        return obj_d


def encode_topic_if_needed(topic):
    """Encode topic if required for specific paho and python version."""
    import sys  # pylint:disable=redefined-outer-name
    if sys.version_info[0] < 3:
        return topic

    # Handle new MQTTMessage version
    version_needs_bytes = packaging.version.parse('1.2.1')
    if mqttcommon.PAHO_VERSION >= version_needs_bytes:
        return topic.encode('utf-8')

    return topic


def mqttmessage(topic, payload):
    """Create message for topic/payload."""
    topic = encode_topic_if_needed(topic)
    msg = MQTTMessage(topic=topic)
    # Handle '_bytes_safe_payload'
    if isinstance(payload, bytearray):
        payload = bytes(payload)

    msg.payload = payload
    return msg


class MQTTClientMock(mqttcommon.MQTTClient):
    """Mock for MQTTClient."""
    ON_MESSAGE_FILTERED = []
    PUBLISH_DELAY = 1
    SUBSCRIBE_DELAY = 1
    CONNECT_DELAY = 1

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Share all callbacks
        self.on_message_filtered = MQTTClientMock.ON_MESSAGE_FILTERED
        self._register_topics_callbacks()

        self.publish_delay = self.PUBLISH_DELAY
        self.subscribe_delay = self.SUBSCRIBE_DELAY
        self.connect_delay = self.CONNECT_DELAY

    @classmethod
    def mock_reset(cls):
        """Cleanup callbacks."""
        del cls.ON_MESSAGE_FILTERED[:]

    def _send_publish(self,  # pylint:disable=too-many-arguments
                      mid, topic, payload=None,
                      qos=0, retain=False, dup=False, info=None):
        """Mock _send_publish function in 'qos == 0' mode."""
        assert info, 'Only info set managed currently'
        assert qos == 0, 'Only qos === 0 managed in tests.'

        self.async_send_message(mqttmessage(topic, payload), info)
        return info

    def async_send_message(self, message, info):
        """Start a thread thatThread run function to publish message."""
        if self.publish_delay:
            threading.Thread(target=self._async_send_message,
                             args=(message, info)).start()
        else:
            self._async_send_message(message, info)

    def _async_send_message(self, message, info):
        """Thread run function to publish message."""
        time.sleep(self.publish_delay)
        self._handle_on_message(message)
        # Set published after 'handled' to ease tests
        info._set_as_published()  # pylint:disable=protected-access

    def subscribe(self, *args, **kwargs):
        # pylint:disable=unused-argument,arguments-differ
        threading.Thread(target=self._subscribe).start()

    def _subscribe(self):
        time.sleep(self.subscribe_delay)
        self.on_subscribe(self, None, None, 0)  # pylint:disable=not-callable

    def connect(self, *args, **kwargs):
        # pylint:disable=unused-argument,arguments-differ
        threading.Thread(target=self._connect).start()

    def _connect(self):
        time.sleep(self.connect_delay)
        self.on_connect(self, None, 0, 0)  # pylint:disable=not-callable

    def loop_start(self):
        pass

    def loop_stop(self, force=False):  # pylint:disable=unused-argument
        pass



#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""Print 'Message' and wait for signal."""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
import signal


def main():
    """Print 'Message' and wait for signal."""
    print('Message')
    signal.pause()


if __name__ == '__main__':
    main()



# -*- coding:utf-8 -*-

"""iotlabapi module tests."""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import argparse

import mock

from iotlabcli.rest import HTTPError
from iotlabmqtt import iotlabapi

from . import TestCaseImproved

# pylint:disable=invalid-name
# patch StringIO: Instance of 'dict' has no 'getvalue' member
# pylint:disable=no-member


class IoTLABAPITestInit(TestCaseImproved):
    """Test IoTLABAPI __init__."""

    def setUp(self):
        self.parser = argparse.ArgumentParser()
        iotlabapi.parser_add_iotlabapi_args(self.parser)

        self.get_experiment = mock.patch(
            'iotlabcli.experiment.get_experiment').start()
        self.get_experiment.return_value = {'state': 'Running'}
        self.stdout = mock.patch('sys.stdout', new_callable=StringIO).start()

    def tearDown(self):
        mock.patch.stopall()

    def test_init_simple(self):
        """Test IoTLABAPI init cases."""
        # Test simple, all args and running experiment

        args = ['--experiment-id', '12345',
                '--iotlab-user', 'us3rn4me',
                '--iotlab-password', 'p4sswd']
        opts = self.parser.parse_args(args)

        api = iotlabapi.IoTLABAPI.from_opts_dict(**vars(opts))

        self.get_experiment.assert_called_with(api.api, 12345, 'state')

        self.assertEqual(api.expid, 12345)
        self.assertEqual(api.api.auth.username, 'us3rn4me')
        self.assertEqual(api.api.auth.password, 'p4sswd')

        out = ''
        self.assertEqual(self.stdout.getvalue(), out)

    @mock.patch('iotlabcli.auth.get_user_credentials')
    def test_init_user_password_authcli(self, get_cred):
        """Test IoTLABAPI user/password from rcfile."""
        get_cred.return_value = ('user', 'password')

        # User and password from rcfile
        args = ['--experiment-id', '12345']
        opts = self.parser.parse_args(args)

        api = iotlabapi.IoTLABAPI.from_opts_dict(**vars(opts))

        self.get_experiment.assert_called_with(api.api, 12345, 'state')
        get_cred.assert_called_with(None, None)
        self.assertEqual(api.expid, 12345)
        self.assertEqual(api.api.auth.username, 'user')
        self.assertEqual(api.api.auth.password, 'password')

        out = ''
        self.assertEqual(self.stdout.getvalue(), out)

    @mock.patch('iotlabcli.auth.get_user_credentials')
    def test_init_no_user_password(self, get_cred):
        """Test IoTLABAPI no user/password Auth fails."""
        get_cred.return_value = (None, None)

        err = HTTPError(None, 401, 'msg', None, None)
        self.get_experiment.side_effect = err

        args = ['--experiment-id', '12345']
        opts = self.parser.parse_args(args)

        self.assertRaises(SystemExit,
                          iotlabapi.IoTLABAPI.from_opts_dict, **vars(opts))

        self.assertTrue(self.get_experiment.called)
        get_cred.assert_called_with(None, None)

        out = (
            'IoT-LAB API not authorized.\n'
            'Provide ``--iotlab-user`` and ``--iotlab-password`` options.\n'
            'Or register them using ``auth-cli --user USERNAME``\n')
        self.assertEqual(self.stdout.getvalue(), out)

    @mock.patch('iotlabcli.helpers.get_current_experiment')
    def test_init_no_experiment(self, get_cur_exp):
        """Test IoTLABAPI no experiment provided."""
        err = ValueError("You have no 'Running' experiment")
        get_cur_exp.side_effect = err

        args = ['--iotlab-user', 'us3rn4me',
                '--iotlab-password', 'p4sswd']
        opts = self.parser.parse_args(args)

        self.assertRaises(SystemExit,
                          iotlabapi.IoTLABAPI.from_opts_dict, **vars(opts))

        self.assertTrue(get_cur_exp.called)
        self.assertFalse(self.get_experiment.called)

        out = "You have no 'Running' experiment\n"
        self.assertEqual(self.stdout.getvalue(), out)

    def test_init_exp_not_running(self):
        """Test IoTLABAPI no experiment provided."""
        self.get_experiment.return_value = {'state': 'Error'}

        args = ['--experiment-id', '12345',
                '--iotlab-user', 'us3rn4me',
                '--iotlab-password', 'p4sswd']
        opts = self.parser.parse_args(args)

        self.assertRaises(SystemExit,
                          iotlabapi.IoTLABAPI.from_opts_dict, **vars(opts))
        self.assertTrue(self.get_experiment.called)

        out = "Experiment 12345 not running 'Error'\n"
        self.assertEqual(self.stdout.getvalue(), out)


class IoTLABAPITest(TestCaseImproved):
    """Test IoTLABAPI."""

    def setUp(self):
        get_experiment = mock.patch(
            'iotlabcli.experiment.get_experiment').start()
        get_experiment.return_value = {'state': 'Running'}

        parser = argparse.ArgumentParser()
        iotlabapi.parser_add_iotlabapi_args(parser)
        args = ['--iotlab-user', 'user',
                '--iotlab-password', 'password',
                '--experiment-id', '12345']
        opts = parser.parse_args(args)
        self.api = iotlabapi.IoTLABAPI.from_opts_dict(**vars(opts))

        # Prevent un-patched calls
        self.api.api = mock.MagicMock(spec_set=True)

    def tearDown(self):
        mock.patch.stopall()

    def test_set_sniffer_channel_localhost(self):
        """Test IoTLABAPI set_sniffer_channel for localhost."""

        # Simple 'localhost' test
        ret = self.api.set_sniffer_channel(11, 'localhost', 30001)
        self.assertEqual(ret, {'30001': ''})

        # Multiple 'localhost' test
        ret = self.api.set_sniffer_channel(11, 'localhost',
                                           30001, 30002, 30003)
        self.assertEqual(ret, {'30001': '', '30002': '', '30003': ''})

    def test_set_sniffer_channel_m3_a8(self):
        """Test IoTLABAPI set_sniffer_channel for m3 and a8."""
        self.api.api.mock_add_spec(['node_profile_load'])

        self.api.api.node_profile_load.return_value = {
            '0': ['m3-42.%s.iot-lab.info' % self.api.HOSTNAME],
        }
        ret = self.api.set_sniffer_channel(11, 'm3', 42)
        self.assertEqual(ret, {'42': ''})

        self.api.api.node_profile_load.return_value = {
            '0': ['m3-42.%s.iot-lab.info' % self.api.HOSTNAME,
                  'm3-66.%s.iot-lab.info' % self.api.HOSTNAME],
        }
        ret = self.api.set_sniffer_channel(11, 'm3', 42, 66)
        self.assertEqual(ret, {'42': '', '66': ''})

        # Command fails, message replaced
        self.api.api.node_profile_load.return_value = {
            '1': ['m3-42.%s.iot-lab.info' % self.api.HOSTNAME],
        }
        ret = self.api.set_sniffer_channel(11, 'm3', 42)
        self.assertEqual(ret, {'42': 'Execution failed on node'})

        self.api.api.node_profile_load.return_value = {
            '0': ['a8-128.%s.iot-lab.info' % self.api.HOSTNAME],
        }
        ret = self.api.set_sniffer_channel(11, 'a8', 128)
        self.assertEqual(ret, {'128': ''})

    def test_set_sniffer_channel_invalid_archi(self):
        """Test IoTLABAPI set_sniffer_channel for not supported archi."""

        ret = self.api.set_sniffer_channel(11, 'des', 1)
        err = 'Create profile failed: Archi: des not currently supported'
        self.assertEqual(ret, {'1': err})

    def test_set_sniffer_channel_fail_set_profile(self):
        """Test IoTLABAPI set_sniffer_channel fail node_profile_load."""
        self.api.api.mock_add_spec(['node_profile_load'])

        self.api.api.node_profile_load.side_effect = RuntimeError('FAILED')
        ret = self.api.set_sniffer_channel(11, 'm3', 42)
        err = "IoT-LAB Request 'profile-load' error: 'FAILED'"
        self.assertEqual(ret, {'42': err})

    def test_node_commands(self):
        """Test IoTLABAPI commands using node_command."""
        self.api.api.mock_add_spec(['node_command'])
        node = 'm3-42.%s.iot-lab.info' % self.api.HOSTNAME
        self.api.api.node_command.return_value = {
            '0': [node]
        }

        ret = self.api.reset('m3', 42)
        self.assertEqual(ret, {'42': ''})
        self.api.api.node_command.assert_called_with('reset', 12345, [node])

        ret = self.api.poweron('m3', 42)
        self.assertEqual(ret, {'42': ''})
        self.api.api.node_command.assert_called_with('start', 12345, [node])

        ret = self.api.poweroff('m3', 42)
        self.assertEqual(ret, {'42': ''})
        self.api.api.node_command.assert_called_with('stop', 12345, [node])

    @mock.patch('iotlabcli.node.node_command')
    def test_update(self, node_command):
        """Test IoTLABAPI update."""
        node = 'm3-42.%s.iot-lab.info' % self.api.HOSTNAME
        node_command.return_value = {'0': [node]}

        ret = self.api.update('tutorial_m3.elf', 'm3', 42)
        self.assertEqual(ret, {'42': ''})
        node_command.assert_called_with(self.api.api, 'update', 12345,
                                        [node], 'tutorial_m3.elf')

    def test_update_no_firmware_file(self):
        """Test IoTLABAPI update without firmware file."""
        ret = self.api.update('/tmp/non/existant/file', 'm3', 42)

        # Use startswith because of python2/python3 string printing compat
        error = ret['42']
        error_start = ("IoT-LAB Request 'update' error: "
                       "'[Errno 2] No such file or directory: ")
        self.assertTrue(error.startswith(error_start))



# -*- coding:utf-8 -*-
"""Tests for IoT-LAB MQTT agents using a real server."""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import os
import time
import subprocess
import unittest

from .. import TestCaseImproved


class IntegrationTestCase(TestCaseImproved):
    """Integration tests which run mosquitto."""
    BROKERPORT = 54321

    @classmethod
    def setUpClass(cls):
        cls.proc = cls.mosquitto_start(port=cls.BROKERPORT)

    @staticmethod
    def mosquitto_start(port, wait=5):
        """Start mosquitto on ``port``.

        :param port: mosquitto listening port
        :param wait: waiting time after starting process to check if running
        """
        mosquitto_conf = os.path.join(os.path.dirname(__file__),
                                      'mosquitto.conf')
        cmd = ['mosquitto', '-p', '%s' % port, '-c', mosquitto_conf]
        try:
            proc = subprocess.Popen(cmd)

            time.sleep(wait)
            if proc.poll() is None:
                return proc
        except OSError:
            pass

        raise unittest.SkipTest('Could not start "mosquitto"')

    @classmethod
    def tearDownClass(cls):
        cls.proc.terminate()
        cls.proc.wait()

    def setUp(self):
        self.socat = {}

    def socat_start(self, port, wait=1):
        """Register new socat on ``port``."""
        self.socat[port] = self._socat_start(port, wait)

    def socat_stop(self, port):
        """Close socat on ``port``."""
        soc = self.socat.pop(port)
        soc.terminate()
        soc.wait()

    @staticmethod
    def _socat_start(port, wait=1):
        """Start socat on ``port``.

        :param port: socat listening port
        :param wait: waiting time after starting process to check if running
        """
        tcp_listen = 'tcp4-listen:{port},reuseaddr,fork'.format(port=port)
        cmd = ['socat', '-', tcp_listen]
        proc = subprocess.Popen(cmd,
                                stdout=subprocess.PIPE,
                                stdin=subprocess.PIPE)
        time.sleep(wait)
        if proc.poll() is not None:
            raise ValueError('Failed starting socat {}'.format(port))

        return proc

    @staticmethod
    def file_path(name):
        """Return testfile ``name`` path.

        Testfiles are in ``files`` directory.
        """
        return os.path.join(os.path.dirname(__file__), 'files', name)



#! /usr/bin/env python
# -*- coding:utf-8 -*-

# This file is a part of IoT-LAB iot-lab-mqtt
# Copyright (C) 2017 INRIA (Contact: admin@iot-lab.info)
# Contributor(s) : see AUTHORS file
#
# This software is governed by the CeCILL license under French law
# and abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# http://www.cecill.info.
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
import itertools
from setuptools import setup, find_packages

PACKAGE = 'iotlabmqtt'
# GPL compatible http://www.gnu.org/licenses/license-list.html#CeCILL
LICENSE = 'CeCILL v2.1'


LONG_DESCRIPTION_FILES = ('README.rst', )


def cat(files, join_str=''):
    """Concatenate `files` content with `join_str` between them."""
    files_content = (open(f).read() for f in files)
    return join_str.join(files_content)


def get_version(package):
    """ Extract package version without importing file
    Importing cause issues with coverage,
        (modules can be removed from sys.modules to prevent this)
    Importing __init__.py triggers importing rest and then requests too

    Inspired from pep8 setup.py
    """
    with open(os.path.join(package, '__init__.py')) as init_fd:
        for line in init_fd:
            if line.startswith('__version__'):
                return eval(line.split('=')[-1])  # pylint:disable=eval-used


INSTALL_REQUIRES = ['paho-mqtt>=1.2', 'future', 'packaging']

ENTRY_POINTS = {
    'console_scripts': [
        # Server script
        'iotlab-mqtt-serial = iotlabmqtt.serial:main',
        'iotlab-mqtt-node = iotlabmqtt.node:main [node]',
        'iotlab-mqtt-radiosniffer = iotlabmqtt.radiosniffer:main [sniffer]',
        'iotlab-mqtt-process = iotlabmqtt.process:main',

        # Client script
        'iotlab-mqtt-clients = iotlabmqtt.clients:main',
    ],
}

EXTRAS_REQUIRE = {
    'node': ['iotlabcli>=2.4.0'],
    'sniffer': ['iotlabcli>=2.4.0'],
}

# Sum all dependecies in 'server'
ALL_EXTRAS_DEPS = list(itertools.chain.from_iterable(EXTRAS_REQUIRE.values()))
EXTRAS_REQUIRE['server'] = ALL_EXTRAS_DEPS

setup(
    name=PACKAGE,
    version=get_version(PACKAGE),
    description='Provide access to IoT-LAB experiments as MQTT agents',
    long_description=cat(LONG_DESCRIPTION_FILES, '\n\n'),
    author='IoT-LAB Team',
    author_email='admin@iot-lab.info',
    url='http://www.iot-lab.info',
    license=LICENSE,
    download_url='http://github.com/iot-lab/iot-lab-mqtt/',
    packages=find_packages(),
    include_package_data=True,
    classifiers=['Development Status :: 3 - Alpha',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3',
                 'Environment :: Console',
                 'Topic :: Utilities', ],
    install_requires=INSTALL_REQUIRES,
    entry_points=ENTRY_POINTS,
    extras_require=EXTRAS_REQUIRE,
)



# -*- coding: utf-8 -*-

r"""IoT-LAB MQTT Serial agent
============================

.. |node|             replace::  ``{serialagenttopic}/{archi}/{num}``
.. |line|             replace::  |node|\ ``/line``
.. |linechannel|      replace::  |node|\ ``/line/data``
.. |linestart|        replace::  |node|\ ``/line/ctl/start``
.. |linestop|         replace::  |node|\ ``/line/ctl/stop``
.. |raw|              replace::  |node|\ ``/raw``
.. |rawchannel|       replace::  |node|\ ``/raw/data``
.. |rawstart|         replace::  |node|\ ``/raw/ctl/start``
.. |rawstop|          replace::  |node|\ ``/raw/ctl/stop``
.. |stop|             replace::  |node|\ ``/ctl/stop``
.. |stopall|          replace::  ``{serialagenttopic}/ctl/stopall``
.. |error_t|          replace::  ``{serialagenttopic}/error/``

.. |channel|          replace::  :ref:`Channel <ChannelTopic>`
.. |request|          replace::  :ref:`Request <RequestTopic>`
.. |error|            replace::  :ref:`Error <ErrorTopic>`

.. |request_topic|    replace::  *{topic}*/**request/{clientid}/{requestid}**
.. |reply_topic|      replace::  *{topic}*/**reply/{clientid}/{requestid}**
.. |in_topic|         replace::  *{topic}*/**in**
.. |out_topic|        replace::  *{topic}*/**out**


Serial agent provides access to the nodes serial link.


Serial agent base topic: ::

   {prefix}/iot-lab/serial/{site}

Every topics from serial agent topics start by this topic prefix

:param prefix: configured prefix
:param site: site where the agent is run


Topics Summary
==============

+-+----------------------------------------------------------------+----------+
| |Topic                                                           | Type     |
+=+================================================================+==========+
|  **Serial agent**                                                           |
+-+----------------------------------------------------------------+----------+
| ``{prefix}/iot-lab/serial/{site}``                                          |
+-+----------------------------------------------------------------+----------+
| ||error_t|                                                       | |error|  |
+-+----------------------------------------------------------------+----------+
| ||stopall|                                                       ||request| |
+-+----------------------------------------------------------------+----------+
|  **Node**                                                                   |
+-+----------------------------------------------------------------+----------+
| ||stop|                                                          ||request| |
+-+----------------------------------------------------------------+----------+
|  **Line redirection**                                                       |
+-+----------------------------------------------------------------+----------+
| ||linestart|                                                     ||request| |
+-+----------------------------------------------------------------+----------+
| ||linestop|                                                      ||request| |
+-+----------------------------------------------------------------+----------+
| ||linechannel|                                                   ||channel| |
+-+----------------------------------------------------------------+----------+
|  **Raw redirection** *(not implemented for the moment)*                     |
+-+----------------------------------------------------------------+----------+


Serial Agent global topics
==========================

Error Topic
-----------

Asynchronous error messages are posted to error topic.
Failure on requests are not duplicated here.

* |error_t|


For format see: :ref:`ErrorTopic`


Stop all redirections
---------------------

Stop all started serial redirections.

+-----------------------------------------------------------------------------+
| ``stopall`` request:                                                        |
|                                                                             |
+============+================================================================+
| Topic:     |    |stopall|                                                   |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Request    | |request_topic|                         | *empty*              |
+------------+-----------------------------------------+----------------------+
| Reply      | |reply_topic|                           | *empty* or error_msg |
+------------+-----------------------------------------+----------------------+


Node topics
===========

.. _stop_redirection:

Stop redirection
----------------

Stop given node serial redirection whatever the mode was.


+-----------------------------------------------------------------------------+
| ``stop`` request:                                                           |
+============+================================================================+
| Topic:     |    |stop|                                                      |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Request    | |request_topic|                         | *empty*              |
+------------+-----------------------------------------+----------------------+
| Reply      | |reply_topic|                           | *empty* or error_msg |
+------------+-----------------------------------------+----------------------+


Line redirection topics
=======================

Topics to access to the node serial port redirection in 'line' mode.
The redirection must first be started in ``line`` mode to have the line output.


Data received from the node is split on newlines characters with newlines
stripped.

As newlines characters are not used in multibytes ``utf-8`` characters,
each line remains, if it was, a valid ``utf-8`` string.
On invalid ``utf-8`` strings sent by the node,
the data will also be invalid in the topic.


Start redirection in *line* mode
--------------------------------

Start one node serial redirection in *line* mode.

+-----------------------------------------------------------------------------+
| ``line/start`` request:                                                     |
+============+================================================================+
| Topic:     |    |linestart|                                                 |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Request    | |request_topic|                         | *empty*              |
+------------+-----------------------------------------+----------------------+
| Reply      | |reply_topic|                           | *empty* or error_msg |
+------------+-----------------------------------------+----------------------+


Stop line redirection
---------------------

Equivalent to :ref:`stop_redirection` added here for completeness.

+-----------------------------------------------------------------------------+
| ``line/stop`` request:                                                      |
+============+================================================================+
| Topic:     |    |linestop|                                                  |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Request    | |request_topic|                         | *empty*              |
+------------+-----------------------------------------+----------------------+
| Reply      | |reply_topic|                           | *empty* or error_msg |
+------------+-----------------------------------------+----------------------+


Line redirection
----------------

Serial redirection sends text lines with newline characters removed (``\n``).

One message is received per line, and when sending a message, the newline
character is automatically added.

+-----------------------------------------------------------------------------+
| **Text line serial redirection**                                            |
+============+================================================================+
| Topic:     |    |linechannel|                                               |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Output     | |out_topic|                             | UTF-8 text line      |
+------------+-----------------------------------------+----------------------+
| Input      | |in_topic|                              | UTF-8 text line      |
+------------+-----------------------------------------+----------------------+


Raw redirection topics
======================

.. Note:: RAW redirection not implemented for the moment
"""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import threading

from . import common
from . import mqttcommon
from . import asyncconnection

PARSER = common.MQTTAgentArgumentParser()


class LineHandler(object):  # pylint:disable=too-few-public-methods
    """Line data handler."""
    def __init__(self, handler):
        self.data = b''
        self.handler = handler

    def __call__(self, data):
        """Call 'handler' on received data line by line."""

        # 'newline' byte never appear in multibyte unicode char
        # so it can be split without decoding
        self.data += data
        lines = self.data.splitlines(True)

        for full_line in lines:
            # strip newline using splitlines
            line = full_line.splitlines()[0]

            # last incomplete line
            if line == full_line:
                self.data = line
                return

            self.handler(line)

        self.data = b''


class SerialConnection(asyncconnection.NodeConnection):
    """Implement serial connection.

    Only overrides ``_address`` to adapt to the serial port.
    """
    PORT = 20000

    def _address(self, archi, num):  # overrides
        """Return socket address for archi/num.

        Hack for 'localhost' to use ``self.num`` as port.
        """
        # pylint:disable=no-else-return
        if archi == 'localhost':
            return archi, int(num)
        else:
            return ('node-%s-%s' % (archi, num), self.PORT)


class Node(object):  # pylint:disable=too-many-instance-attributes
    """Node resource managing connection, handling commands and events.

    :param archi: node architecture, or localhost
    :param num: node number
    :param closed_cb: callback for connection close
    :type closed_cb: Callable[[Node], None]
    :param error_cb: callback for asynchronous errors
    :type error_cb: Callable[[Node, str], None]
    """

    STATES = ('closed', 'linestarting', 'line')

    def __init__(self, archi, num,  # pylint:disable=too-many-arguments
                 closed_cb, error_cb, asyncoreservice=None):
        self.host = self.hostname(archi, num)
        self.closed_cb = closed_cb
        self.error_cb = error_cb

        self.state = 'closed'
        self.reply_publisher = None
        self.connection = SerialConnection(archi, num, self.conn_event_handler,
                                           service=asyncoreservice)

        # Required Rlock, connection socket errors calls event_handler('close')
        self._rlock = threading.RLock()

    @staticmethod
    def hostname(archi, num):
        """Return hostname for archi, num."""
        return archi, num

    @staticmethod
    def host_str(archi, num):
        """Host string."""
        return '%s-%s' % (archi, num)

    @property
    def state(self):
        """Node state."""
        return self._state

    @state.setter
    def state(self, value):
        """Node state, check ``value`` is a valid state."""
        assert value in self.STATES
        self._state = value  # pylint:disable=attribute-defined-outside-init

    def _reply_request(self, reply, newstate=None):
        """Reply current request and remove publisher.

        If ``newstate`` is provided, update current state.
        """
        self.state = newstate or self.state
        self.reply_publisher(reply.encode('utf-8'))
        self.reply_publisher = None

    @common.synchronized('_rlock')
    def close(self):
        """Close connection and state."""
        self._close()
        return b''  # in case of a request

    def _close(self):
        """Set 'closed' state, resets data_handler and call ``closed_cb``."""
        previous_state = self.state
        self.connection.close()
        self.connection.data_handler = None
        self.state = 'closed'
        self.closed_cb(self)
        return previous_state

    @common.synchronized('_rlock')
    def conn_event_handler(self, event):
        """Handler for connection events."""
        event_handler = {
            'connect': self._event_connect,
            'error': self._event_error,
            'close': self._event_close,
        }
        return event_handler[event]()

    def _event_connect(self):
        if self.state == 'linestarting':
            self._reply_request('', newstate='line')
            return

        raise Exception('Got connect event in invalid state %s' % self.state)

    def _event_error(self):
        # Received other events exceptions
        error = common.traceback_error()
        previous_state = self._close()

        if previous_state == 'linestarting':
            self._reply_request('Connection failed: %s' % error)
            return

        self.error_cb(self, error)

    def _event_close(self):
        # Jumps to event error
        raise Exception('Connection closed in state %s' % self.state)

    @common.synchronized('_rlock')
    def req_linestart(self, reply_publisher, line_handler):
        """Request to start line."""

        if self.state == 'line':
            return b''

        if self.state != 'closed':
            err = "Error: 'linestart' in mode %s. Wait or stop it first"
            return (err % self.state).encode('utf-8')

        # Start line mode and register 'line_handler'
        self.state = 'linestarting'
        self.connection.data_handler = line_handler

        # Async answer
        self.reply_publisher = reply_publisher
        self.connection.start()
        return None

    @common.synchronized('_rlock')
    def lineinput(self, payload):
        """Send ``payload`` with a newline to the node connection."""
        if self.state != 'line':
            raise ValueError("lineinput while not in 'line' mode")

        line = payload + b'\n'
        self.connection.send(line)


class MQTTAggregator(object):
    """Aggregator implementation for MQTT."""

    AGENTTOPIC = 'iot-lab/serial/{site}'
    TOPICS = {
        'node': '{archi}/{num}',
        'line': '{archi}/{num}/line',
    }

    HOSTNAME = common.hostname()

    def __init__(self, client, prefix=''):
        super().__init__()

        staticfmt = {'site': self.HOSTNAME}
        _topics = mqttcommon.generate_topics_dict(self.TOPICS, prefix,
                                                  self.AGENTTOPIC, staticfmt)

        self.nodes = {}
        self.asyncore = asyncconnection.AsyncoreService()

        self.topics = {
            'node': mqttcommon.NullTopic(_topics['node']),
            'line': mqttcommon.ChannelServer(_topics['line'],
                                             callback=self.cb_lineinput),
            'linestart': mqttcommon.RequestServer(
                _topics['line'], 'start', callback=self.cb_linestart),
            'linestop': mqttcommon.RequestServer(
                _topics['line'], 'stop', callback=self.cb_stop),

            'stop': mqttcommon.RequestServer(
                _topics['node'], 'stop', callback=self.cb_stop),

            'stopall': mqttcommon.RequestServer(
                _topics['agenttopic'], 'stopall', callback=self.cb_stopall),

            'error': mqttcommon.ErrorServer(_topics['agenttopic']),
        }

        self.client = client
        self.client.topics = list(self.topics.values())

    def error(self, topic, message):
        """Publish error that happend on topic."""
        self.topics['error'].publish_error(self.client, topic,
                                           message.encode('utf-8'))

    def _node_error(self, node, message):
        archi, num = node.host
        topic = self.topics['node'].topic.format(archi=archi, num=num)
        self.error(topic, message)

    def cb_lineinput(self, message, archi, num):
        """Write message to node."""
        host = Node.hostname(archi, num)
        try:
            self.nodes[host].lineinput(message.payload)
        except KeyError:
            self.error(message.topic, 'Non connected node {}'.format(
                Node.host_str(archi, num)))

    def cb_linestart(self, message, archi, num):
        """Start node redirection in 'line' mode.

        Create a new node if it does not currently exists.
        """
        new_node = Node(archi, num, self._node_closed_cb, self._node_error,
                        asyncoreservice=self.asyncore)
        node = self.nodes.setdefault(Node.hostname(archi, num), new_node)

        line_handler = self._line_handler(archi, num)
        return node.req_linestart(message.reply_publisher, line_handler)

    def _line_handler(self, archi, num):
        """Line handler for node ``archi``, ``num``.

        Publish the message to the correct topic for ``archi``, ``num``.
        """
        channel = self.topics['line']
        publisher = channel.output_publisher(self.client, archi=archi, num=num)
        return LineHandler(publisher)

    def _node_closed_cb(self, node):
        """Remove closed node."""
        self.nodes.pop(node.host, None)

    def cb_stop(self, message, archi, num):
        """Stop node redirection."""
        try:
            return self.nodes[Node.hostname(archi, num)].close()
        except KeyError:
            return b''

    def cb_stopall(self, message):
        """Stop nodes redirection."""
        self._stop_all_nodes()
        return b''

    # Agent running

    def run(self):
        """Run agent."""
        try:
            self.start()
            common.wait_sigint()
        finally:
            self.stop()

    def start(self):
        """Start Agent."""
        self.asyncore.start()
        self.client.start()

    def stop(self):
        """Stop agent."""
        self.client.stop()
        self._stop_all_nodes()
        self.asyncore.stop()

    def _stop_all_nodes(self):
        """Close all nodes connections."""
        for node in list(self.nodes.values()):
            node.close()

    @classmethod
    def from_opts_dict(cls, prefix, **kwargs):
        """Create class from argparse entries."""
        client = mqttcommon.MQTTClient.from_opts_dict(**kwargs)
        return cls(client, prefix)


def main():
    """Run serial agent."""
    opts = PARSER.parse_args()
    aggr = MQTTAggregator.from_opts_dict(**vars(opts))
    aggr.run()



# -*- coding: utf-8 -*-

r"""IoT-LAB MQTT Node agent
===========================

Node Agent provides access to the IoT-LAB nodes interactions:

* update the node firmware
* reset the CPU
* power node ON and OFF

Node agent base topic: ::

   {prefix}/iot-lab/node/{site}

Every topics from node agent topics start by this topic prefix

:param prefix: configured prefix
:param site: site where the agent is run


Topics Summary
==============

.. |node|             replace::  ``{nodeagenttopic}/{archi}/{num}``

.. |update|           replace:: |node|\ ``/ctl/update``
.. |reset|            replace:: |node|\ ``/ctl/reset``
.. |poweron|          replace:: |node|\ ``/ctl/poweron``
.. |poweroff|         replace:: |node|\ ``/ctl/poweroff``
.. |error_t|          replace::  ``{nodeagenttopic}/error/``


.. |request|          replace::  :ref:`Request <RequestTopic>`
.. |error|            replace::  :ref:`Error <ErrorTopic>`

.. |request_topic|    replace::  *{topic}*/**request/{clientid}/{requestid}**
.. |reply_topic|      replace::  *{topic}*/**reply/{clientid}/{requestid}**

+-+---------------------------------------------------------------+-----------+
| |Topic                                                          | Type      |
+=+===============================================================+===========+
|  **Node agent**                                                             |
+-+---------------------------------------------------------------+-----------+
| ``{prefix}/iot-lab/node/{site}``                                            |
+-+---------------------------------------------------------------+-----------+
| ||error_t|                                                      | |error|   |
+-+---------------------------------------------------------------+-----------+
|  **Node**                                                                   |
+-+---------------------------------------------------------------+-----------+
| ||update|                                                       ||request|  |
+-+---------------------------------------------------------------+-----------+
| ||reset|                                                        ||request|  |
+-+---------------------------------------------------------------+-----------+
| ||poweron|                                                      ||request|  |
+-+---------------------------------------------------------------+-----------+
| ||poweroff|                                                     ||request|  |
+-+---------------------------------------------------------------+-----------+


Node Agent global topics
========================

Error Topic
-----------

Asynchronous error messages are posted to error topic.
Failure on requests are not duplicated here.

* |error_t|


For format see: :ref:`ErrorTopic`


Node topics
===========

Requests to interact with nodes.


.. |postnotespace| raw:: html

   <p></p>

Update firmware
---------------

Update node with given firmware. If no firmware is provided ``empty``,
default ``idle`` firmware is used instead.

:archi: supported ``m3``

|postnotespace|

+-----------------------------------------------------------------------------+
| ``update`` request:                                                         |
+============+================================================================+
| Topic:     |    |update|                                                    |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Request    | |request_topic|                         |``Firmware binary`` or|
|            |                                         | *empty* for ``idle`` |
+------------+-----------------------------------------+----------------------+
| Reply      | |reply_topic|                           | *empty* or error_msg |
+------------+-----------------------------------------+----------------------+


Reset cpu
---------

Reset node CPU to restart firmware execution.

:archi: supported ``m3``

|postnotespace|

+-----------------------------------------------------------------------------+
| ``reset`` request:                                                          |
|                                                                             |
+============+================================================================+
| Topic:     |    |reset|                                                     |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Request    | |request_topic|                         | *empty*              |
+------------+-----------------------------------------+----------------------+
| Reply      | |reply_topic|                           | *empty* or error_msg |
+------------+-----------------------------------------+----------------------+


Power OFF Node
--------------

Power OFF node.

:Note: Nodes are powered ON by default.
:archi: supported ``m3``, ``a8``

|postnotespace|

+-----------------------------------------------------------------------------+
| ``poweroff`` request:                                                       |
|                                                                             |
+============+================================================================+
| Topic:     |    |poweroff|                                                  |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Request    | |request_topic|                         | *empty*              |
+------------+-----------------------------------------+----------------------+
| Reply      | |reply_topic|                           | *empty* or error_msg |
+------------+-----------------------------------------+----------------------+


Power ON Node
-------------

Power ON node if it has been powered OFF.
Powering ON an already powered ON does nothing.

:Note: Nodes are powered ON by default.
:archi: supported ``m3``, ``a8``

|postnotespace|

+-----------------------------------------------------------------------------+
| ``poweron`` request:                                                        |
|                                                                             |
+============+================================================================+
| Topic:     |    |poweron|                                                   |
+------------+-----------------------------------------+----------------------+
|**Message** | **Topic**                               | **Payload**          |
+------------+-----------------------------------------+----------------------+
| Request    | |request_topic|                         | *empty*              |
+------------+-----------------------------------------+----------------------+
| Reply      | |reply_topic|                           | *empty* or error_msg |
+------------+-----------------------------------------+----------------------+
"""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import os
import threading
import tempfile
import contextlib

from . import common
from . import iotlabapi
from . import mqttcommon

PARSER = common.MQTTAgentArgumentParser()
iotlabapi.parser_add_iotlabapi_args(PARSER)


@contextlib.contextmanager
def _firmware_file(data):
    """Context manager that yields a temporary file with ``data``."""
    sha1_suffix = '--sha1:%s' % common.short_hash(data)

    with tempfile.NamedTemporaryFile('w+b', suffix=sha1_suffix) as tmpfile:
        tmpfile.write(data)
        # Ensure file is flushed as it should be readable before close
        tmpfile.flush()
        yield tmpfile.name


class MQTTNodeAgent(object):
    """Node Agent implementation for MQTT."""
    AGENTTOPIC = 'iot-lab/node/{site}'
    TOPICS = {
        'node': '{archi}/{num}',
    }
    HOSTNAME = common.hostname()

    def __init__(self, client, prefix='', iotlab_api=None):
        assert iotlab_api
        super().__init__()

        staticfmt = {'site': self.HOSTNAME}
        _topics = mqttcommon.generate_topics_dict(self.TOPICS, prefix,
                                                  self.AGENTTOPIC, staticfmt)

        self.topics = {
            'reset': mqttcommon.RequestServer(_topics['node'], 'reset',
                                              callback=self.cb_reset),
            'update': mqttcommon.RequestServer(_topics['node'], 'update',
                                               callback=self.cb_update),
            'poweron': mqttcommon.RequestServer(_topics['node'], 'poweron',
                                                callback=self.cb_poweron),
            'poweroff': mqttcommon.RequestServer(_topics['node'], 'poweroff',
                                                 callback=self.cb_poweroff),

            'error': mqttcommon.ErrorServer(_topics['agenttopic']),
        }

        self.iotlabapi = iotlab_api

        self.client = client
        self.client.topics = list(self.topics.values())

    def cb_reset(self, message, archi, num):
        """Reset node cpu."""
        args = (self.iotlabapi.reset, message.reply_publisher, archi, num)
        threading.Thread(target=self._thread_command, args=args).start()
        return None

    def cb_update(self, message, archi, num):
        """Update node firmware."""
        args = (message.payload, message.reply_publisher, archi, num)
        threading.Thread(target=self._thread_update, args=args).start()
        return None

    def _thread_update(self, firmware, reply_publisher, archi, num):
        if not firmware:
            ret_dict = self.iotlabapi.update_idle(archi, num)
        else:
            with _firmware_file(firmware) as firmware_path:
                ret_dict = self.iotlabapi.update(firmware_path, archi, num)

        ret = ret_dict[str(num)]
        reply_publisher(ret.encode('utf-8'))

    @staticmethod
    def _idle_m3_firmware():
        """Return m3 idle firmware."""
        path = os.path.join(os.path.dirname(__file__), 'static', 'idle_m3.elf')
        with open(path, 'rb') as firmware:
            return firmware.read()

    def cb_poweron(self, message, archi, num):
        """Power ON node."""
        args = (self.iotlabapi.poweron, message.reply_publisher, archi, num)
        threading.Thread(target=self._thread_command, args=args).start()
        return None

    def cb_poweroff(self, message, archi, num):
        """Power OFF node."""
        args = (self.iotlabapi.poweroff, message.reply_publisher, archi, num)
        threading.Thread(target=self._thread_command, args=args).start()
        return None

    @staticmethod
    def _thread_command(func, reply_publisher, archi, num):
        ret_dict = func(archi, num)
        ret = ret_dict[str(num)]
        reply_publisher(ret.encode('utf-8'))

    # Agent running

    def run(self):
        """Run agent."""
        try:
            self.start()
            common.wait_sigint()
        finally:
            self.stop()

    def start(self):
        """Start Agent."""
        self.client.start()

    def stop(self):
        """Stop agent."""
        self.client.stop()

    @classmethod
    def from_opts_dict(cls, prefix, **kwargs):
        """Create class from argparse entries."""
        api = iotlabapi.IoTLABAPI.from_opts_dict(**kwargs)
        client = mqttcommon.MQTTClient.from_opts_dict(**kwargs)
        return cls(client, prefix, iotlab_api=api)


def main():
    """Run radio sniffer agent."""
    opts = PARSER.parse_args()
    aggr = MQTTNodeAgent.from_opts_dict(**vars(opts))
    aggr.run()



# -*- coding:utf-8 -*-
"""Integration tests for IoT-LAB MQTT serial client and server."""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import contextlib
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import mock

from iotlabmqtt import serial
from iotlabmqtt.clients import serial as serial_client

from . import IntegrationTestCase
from .. import TestCaseImproved


class SerialIntegrationTest(IntegrationTestCase):
    """Test serial client and server using a broker."""

    def setUp(self):
        super().setUp()
        for port in range(20001, 20003):
            self.socat_start(port)

    def tearDown(self):
        for port in list(self.socat):
            self.socat_stop(port)

    @staticmethod
    @contextlib.contextmanager
    def start_client_and_server(brokerport):
        """Start serial client and server context manager.

        Yields client and stdout mock.
        """
        args = ['localhost', '--broker-port', '%s' % brokerport,
                '--prefix', 'serial/test/prefix']
        opts = serial.PARSER.parse_args(args)
        server = serial.MQTTAggregator.from_opts_dict(**vars(opts))
        server.start()

        try:
            args = ['localhost', '--broker-port', '%s' % brokerport,
                    '--prefix', 'serial/test/prefix',
                    '--site', server.HOSTNAME]
            opts = serial_client.PARSER.parse_args(args)
            client = serial_client.SerialShell.from_opts_dict(**vars(opts))
            client.start()

            try:
                with mock.patch('sys.stdout', new_callable=StringIO) as stdout:
                    yield client, stdout
            finally:
                client.stop()
        finally:
            server.stop()

    def test_serial_agent_one_node(self):
        """Test serial agent normal cases."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_serial_agent_one_node(client, stdout)

    def _test_serial_agent_one_node(self, client, stdout):
        port = 20001
        soc = self.socat[port]

        # Write non nonnected line
        first = 'err_not_conn'
        client.onecmd('linewrite localhost %u %s' % (port, first))

        err = ('SERIAL ERROR: localhost/20001/line/data/in: '
               'Non connected node localhost-20001\n'
               '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, err, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # Start line
        client.onecmd('linestart localhost %u' % port)

        # Start line again
        client.onecmd('linestart localhost %u' % port)

        # No message == success for both
        self.assertEqual(stdout.getvalue(), '')

        # Write to node
        second = 'wrïttén'
        client.onecmd('linewrite localhost %u %s' % (port, second))

        out = soc.stdout.read(len((second + '\n').encode('utf-8')))
        self.assertEqual(out, (second + '\n').encode('utf-8'))

        # Node answers
        second_ret = 'ánßwër'
        soc.stdin.write((second_ret + '\n').encode('utf-8'))
        soc.stdin.flush()

        answer = ('line_handler(localhost-20001): ánßwër\n'
                  '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, answer, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # Node sends not utf-8 data
        invalid_bytes = b'this: \xc3\x28 is invalid\n'
        soc.stdin.write(invalid_bytes)
        soc.stdin.flush()

        answer = ('line_handler(localhost-20001): this: �( is invalid\n'
                  '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, answer, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # Stop node
        client.onecmd('stop localhost %u' % port)
        self.assertEqual(stdout.getvalue(), '')
        stdout.seek(0)
        stdout.truncate(0)

        # Correctly stopped
        third = 'err_stopped'
        client.onecmd('linewrite localhost %u %s' % (port, third))

        err = ('SERIAL ERROR: localhost/20001/line/data/in: '
               'Non connected node localhost-20001\n'
               '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, err, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # Can stop another time without error
        client.onecmd('stop localhost %u' % port)
        self.assertEqual(stdout.getvalue(), '')

        # Stop all nodes (but no nodes in fact)
        client.onecmd('stopall')
        self.assertEqual(stdout.getvalue(), '')

    def test_serial_agent_no_port(self):
        """Test connecting on non connected node."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_serial_agent_no_port(client, stdout)

    def _test_serial_agent_no_port(self, client, stdout):
        port = 20000
        assert port not in self.socat, "port 20000 is used"

        # Start line non existing node
        client.onecmd('linestart localhost %u' % port)

        # Error message
        err = 'Connection failed: [Errno 111] Connection refused\n'
        self.assertEqual(stdout.getvalue(), err)
        stdout.seek(0)
        stdout.truncate(0)

    def test_serial_agent_dns_fail(self):
        """Test connecting on non existing dns node."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_serial_agent_dns_fail(client, stdout)

    def _test_serial_agent_dns_fail(self, client, stdout):
        """This dns error is raised directly by connect.

        So it triggers a different path than an async failure.
        """

        # Start line non existing dns entry (even on IoT-LAB, raises DNS fail)
        client.onecmd('linestart m3 1234')

        # Error message
        error_start = 'Connection failed: [Errno -'
        # -5] No address associated with hostname')
        # got a different message on ArchLinux
        # -2] Name or service not known
        self.assertTrue(stdout.getvalue().startswith(error_start))
        stdout.seek(0)
        stdout.truncate(0)

    def test_serial_agent_conn_stop(self):
        """Connection stops on one node, and stopall."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_serial_agent_conn_stop(client, stdout)

    def _test_serial_agent_conn_stop(self, client, stdout):
        port = 20001

        for port in self.socat:
            # Start line
            client.onecmd('linestart localhost %u' % port)
        self.assertEqual(stdout.getvalue(), '')

        # Break connection and error from disconnect
        self.socat_stop(port)

        err = ('SERIAL ERROR: localhost/20002: '
               'Connection closed in state line\n'
               '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, err, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # Restart socat
        self.socat_start(port)

        # Stop all nodes with nodes
        client.onecmd('stopall')
        self.assertEqual(stdout.getvalue(), '')


@mock.patch('sys.stdout', new_callable=StringIO)
class SerialClientErrorTests(TestCaseImproved):
    """Test SerialClient Parsing errors."""
    def setUp(self):
        args = ['localhost', '--broker-port', '%s' % 54321,
                '--prefix', 'serial/test/prefix',
                '--site', serial.MQTTAggregator.HOSTNAME]
        opts = serial_client.PARSER.parse_args(args)
        self.client = serial_client.SerialShell.from_opts_dict(**vars(opts))
        # Publish should crash
        self.client.publish = None

    def test_linestart(self, stdout):
        """Test linestart parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: linestart ARCHI NUM\n'
               '  ARCHI: m3/a8\n'
               '  NUM:   node num\n')

        # Missing port
        self.client.onecmd('linestart localhost')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Invalid port
        self.client.onecmd('linestart localhost abc')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Too many arguments
        self.client.onecmd('linestart localhost 123 haha')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_stop(self, stdout):
        """Test stop parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: stop ARCHI NUM\n'
               '  ARCHI: m3/a8\n'
               '  NUM:   node num\n')

        # Missing port
        self.client.onecmd('stop localhost')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Invalid port
        self.client.onecmd('stop localhost abc')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Too many arguments
        self.client.onecmd('stop localhost 123 haha')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_linewrite(self, stdout):
        """Test linewrite parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: linewrite ARCHI NUM MESSAGE\n'
               '  ARCHI: m3/a8\n'
               '  NUM:   node num\n'
               '  MESSAGE: Message line to send\n')

        # Missing message
        self.client.onecmd('linewrite localhost 123')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Invalid port
        self.client.onecmd('linewrite localhost abc message')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_stopall(self, stdout):
        """Test stopall help."""
        hlp = 'stopall\n'

        # cannot be done from command, just call it
        self.client.onecmd('help stopall')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)



# -*- coding: utf-8 -*-

"""IoT-LAB MQTT Serial agent client"""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import iotlabmqtt.serial
from iotlabmqtt import common
from iotlabmqtt import mqttcommon

from . import common as clientcommon


PARSER = common.MQTTAgentArgumentParser()
clientcommon.parser_add_site_arg(PARSER)


class SerialShell(clientcommon.CmdShell):
    """Serial Agent Shell.

    :param client: mqttclient instance
    :param prefix: topics prefix
    :param site: agent site
    """
    LINESTART_USAGE = ('linestart ARCHI NUM\n'
                       '  ARCHI: m3/a8\n'
                       '  NUM:   node num\n')
    LINEWRITE_USAGE = ('linewrite ARCHI NUM MESSAGE\n'
                       '  ARCHI: m3/a8\n'
                       '  NUM:   node num\n'
                       '  MESSAGE: Message line to send\n')
    STOP_USAGE = ('stop ARCHI NUM\n'
                  '  ARCHI: m3/a8\n'
                  '  NUM:   node num\n')

    STOPALL_USAGE = 'stopall\n'

    SERVER = iotlabmqtt.serial.MQTTAggregator

    def __init__(self, client, prefix, site=None):
        assert site is not None
        super().__init__()

        clientid = clientcommon.clientid('serialclient')

        staticfmt = {'site': site}
        _topics = mqttcommon.generate_topics_dict(
            self.SERVER.TOPICS, prefix, self.SERVER.AGENTTOPIC, staticfmt)

        _print_wrapper = self.async_print_handle_readlinebuff()
        line_cb = _print_wrapper(self.line_handler)
        error_cb = _print_wrapper(self.error_cb)

        self.topics = {
            'line': mqttcommon.ChannelClient(_topics['line'], line_cb),
            'linestart': mqttcommon.RequestClient(
                _topics['line'], 'start', clientid=clientid),
            'linestop': mqttcommon.RequestClient(
                _topics['line'], 'stop', clientid=clientid),

            'stop': mqttcommon.RequestClient(
                _topics['node'], 'stop', clientid=clientid),

            'stopall': mqttcommon.RequestClient(
                _topics['agenttopic'], 'stopall', clientid=clientid),

            'error': mqttcommon.ErrorClient(_topics['agenttopic'],
                                            callback=error_cb),
        }

        self.client = client
        self.client.topics = list(self.topics.values())

    def error_cb(self, message, relative_topic):  # pylint:disable=no-self-use
        """Callback on 'error' topic."""
        msg = message.payload.decode('utf-8')
        print('SERIAL ERROR: %s: %s' % (relative_topic, msg))

    @classmethod
    def from_opts_dict(cls, prefix, site, **kwargs):
        """Create class from argparse entries."""
        client = mqttcommon.MQTTClient.from_opts_dict(**kwargs)
        return cls(client, prefix=prefix, site=site)

    # # # # #
    # line  #
    # # # # #
    def line_handler(self, message, archi, num):  # pylint:disable=no-self-use
        """Handle line received from nodes."""
        line = message.payload.decode('utf-8', 'replace')
        print('line_handler(%s-%s): %s' % (archi, num, line))

    # # # # # # #
    # linestart #
    # # # # # # #
    def do_linestart(self, arg):
        """Start line mode to given node: ARCHI NUM"""
        archi, num = self.cmd_split(arg)
        num = int(num)

        self._do_linestart(archi, num)

    def _do_linestart(self, archi, num):
        topic = self.topics['linestart']

        ret = topic.request(self.client, b'', timeout=5,
                            archi=archi, num=num)
        if ret:
            raise RuntimeError(ret.decode('utf-8'))

    def help_linestart(self):
        """Help linestart command."""
        print(self.LINESTART_USAGE, end='')

    # # # # # # #
    # linewrite #
    # # # # # # #
    def do_linewrite(self, arg):
        """Write line to given node: ARCHI NUM MESSAGE."""
        archi, num, message = self.cmd_split(arg, 2)
        num = int(num)

        payload = message.encode('utf-8')
        self.topics['line'].send(self.client, payload, archi=archi, num=num)

    def help_linewrite(self):
        """Help linewrite command."""
        print(self.LINEWRITE_USAGE, end='')

    # # # # #
    # stop  #
    # # # # #
    def do_stop(self, arg):
        """Stop node redirection: ARCHI NUM."""
        archi, num = self.cmd_split(arg)
        num = int(num)

        topic = self.topics['stop']
        ret = topic.request(self.client, b'', timeout=5,
                            archi=archi, num=num)
        if ret:
            raise RuntimeError(ret.decode('utf-8'))

    def help_stop(self):
        """Help stop command."""
        print(self.STOP_USAGE, end='')

    # # # # # #
    # stopall #
    # # # # # #
    def do_stopall(self, _):
        """Stop all nodes redirection."""
        topic = self.topics['stopall']
        ret = topic.request(self.client, b'', timeout=5)
        if ret:
            raise RuntimeError(ret.decode('utf-8'))

    def help_stopall(self):
        """Help stopall command."""
        print(self.STOPALL_USAGE, end='')

    def run(self):
        """Run client and shell."""
        self.start()
        try:
            self.cmdloop()
        except KeyboardInterrupt:
            pass
        self.stop()

    def start(self):
        """Start Agent."""
        self.client.start()

    def stop(self):
        """Stop agent."""
        self.client.stop()


def main(opts=None):
    """SerialAgent shell client.

    :param opts: If provided, don't parse command line but use it instead
    :type opts: argparse Namespace object
    """
    opts = opts or PARSER.parse_args()
    shell = SerialShell.from_opts_dict(**vars(opts))
    shell.run()



# -*- coding:utf-8 -*-

"""MQTTCommon module tests."""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import os.path
import threading

import mock
from iotlabmqtt import mqttcommon
from iotlabmqtt.clients import common as clientcommon
from . import mqttclient_mock
from . import TestCaseImproved


def wrap_mock(func):
    """Call 'func' mock.

    Should be a real function for 'functools.wraps' callback
    (issues while setting '__name__' in a python2/python3 compatible way).
    """
    def _cb_wrapper(*args, **kwargs):
        return func(*args, **kwargs)

    return _cb_wrapper


class AgentTest(TestCaseImproved):
    """Setup and cleanup Agent mock."""

    def setUp(self):
        mqttclient_mock.MQTTClientMock.mock_reset()

    def tearDown(self):
        mqttclient_mock.MQTTClientMock.mock_reset()


class MQTTClientTest(AgentTest):
    """Test MQTTClient using wrapper mock."""
    def test_mqttagent_start_stop(self):
        """Test agent start/stop and topics registering."""
        callback = mock.Mock()
        topics = {'one': mqttcommon.Topic('a/b/c', wrap_mock(callback)),
                  'two': mqttcommon.Topic('1/2/3'),
                  # No subscribe for topic 'three'
                  'three': mqttcommon.NullTopic('not_subscribe')}

        agent = mqttclient_mock.MQTTClientMock('localhost', 1883,
                                               list(topics.values()))
        with mock.patch('iotlabmqtt.mqttcommon.print') as stdout:
            agent.start()

        self.assertEqual(agent.topics, list(topics.values()))
        self.assertEqual(agent.on_message_filtered,
                         [('a/b/c', topics['one'].callback)])

        # Not subscribed to 'three' as 'subscribe_topic' is None
        stdout.assert_has_calls([mock.call('Subscribing to: a/b/c'),
                                 mock.call('Subscribing to: 1/2/3')],
                                any_order=True)

        agent.stop()

    def test_mqttagent_start_timeout(self):
        """Test starting agent timeout."""
        agent = mqttclient_mock.MQTTClientMock('localhost', 1883)
        agent.connect_delay = 15
        self.assertRaises(RuntimeError, agent.start)


class TopicTest(AgentTest):
    """Test Topic class."""

    def test_topic(self):
        """Test valid a topic functions."""
        topicname = 'a/b/{val}/{another}'
        callback = mock.Mock()

        topic = mqttcommon.Topic(topicname, callback=wrap_mock(callback))

        self.assertEqual(topic.topic, 'a/b/{val}/{another}')
        self.assertEqual(topic.fields, ['val', 'another'])
        self.assertEqual(topic.subscribe_topic, 'a/b/+/+')
        self.assertEqual(topic.match_re.pattern,
                         'a/b/(?P<val>[^/]+)/(?P<another>[^/]+)')

        values = topic.fields_values('a/b/first/second')
        self.assertEqual(values, {'val': 'first', 'another': 'second'})

    def test_topic_callback(self):
        """Test calling topic callback."""
        topicname = 'a/b/{val}/{another}'

        callback = mock.Mock()
        topic = mqttcommon.Topic(topicname, callback=wrap_mock(callback))

        mqttc = mock.Mock()
        obj = mock.Mock()

        msg_topic = 'a/b/value/argument'
        msg_topic = mqttclient_mock.encode_topic_if_needed(msg_topic)
        msg = mqttcommon.mqtt.MQTTMessage(topic=msg_topic)

        topic.callback(mqttc, obj, msg)

        self.assertTrue(callback.called)
        callback.assert_called_with(msg, val='value', another='argument')

    def test_topic_none_calback(self):
        """Test callback None even after wrapping."""
        callback = None
        topic = mqttcommon.Topic('to/ic', callback=callback)
        self.assertIsNone(topic.callback)


class RequestTest(AgentTest):
    """Test Request classes."""

    def test_request_topic(self):
        """Test RequestTopic."""
        clientid = clientcommon.clientid('testrequest')

        topicname = '{archi}/{num}/line'
        requesttopic = '{archi}/{num}/line/ctl/start/request/%s/{requestid}'
        requesttopic %= clientid
        replytopic = '{archi}/{num}/line/ctl/start/reply/%s/{requestid}'
        replytopic %= clientid

        # Verify topics values
        self.assertEqual(
            mqttcommon.RequestTopic.request_topic(topicname, 'start',
                                                  clientid=clientid),
            requesttopic)
        self.assertEqual(
            mqttcommon.RequestTopic.reply_topic(topicname, 'start',
                                                clientid=clientid),
            replytopic)

        # Request/reply conversion
        self.assertEqual(mqttcommon.RequestTopic.reply_topic_from_request(
            requesttopic), replytopic)
        self.assertEqual(mqttcommon.RequestTopic.request_topic_from_reply(
            replytopic), requesttopic)

        # Check fields values cleanup
        fields_values = {'archi': 'm3', 'num': '1',
                         'clientid': 'CLIENTID', 'requestid': 'REQUESTID'}
        self.assertEqual(
            mqttcommon.RequestTopic.clean_callback_fields(fields_values),
            {'archi': 'm3', 'num': '1'})

    def test_request_topics(self):
        """Test Request Topics."""
        clientid = clientcommon.clientid('testrequest')
        topicname = '{archi}/{num}/line'

        server_cb = mock.Mock()

        client_topics = {'linestart': mqttcommon.RequestClient(
            topicname, 'start', clientid=clientid)}
        server_topics = {'linestart': mqttcommon.RequestServer(
            topicname, 'start', wrap_mock(server_cb))}

        client = mqttclient_mock.MQTTClientMock('localhost', 1883,
                                                list(client_topics.values()))
        server = mqttclient_mock.MQTTClientMock('localhost', 1883,
                                                list(server_topics.values()))

        # Simple request answering
        req_topic = mqttcommon.common.topic_lazyformat(
            server_topics['linestart'].topic,
            archi='m3', num='1', clientid=clientid)

        server_cb.return_value = b'result'
        request_msg = mqttclient_mock.mqttmessage(
            req_topic.format(requestid=1), b'queryone')
        ret = client_topics['linestart'].request(client, b'queryone',
                                                 archi='m3', num='1')
        server_cb.assert_called_with(request_msg, archi='m3', num='1')
        server_cb.reset_mock()
        self.assertEqual(ret, b'result')

        # Async answer
        def _server_cb(msg, archi, num):
            thr = threading.Thread(target=__server__cb,
                                   args=(msg.reply_publisher, b'responsetwo'))
            thr.start()
            return None

        def __server__cb(publisher, payload):
            import time
            time.sleep(1)
            publisher(payload)

        server_cb.side_effect = _server_cb
        request_msg = mqttclient_mock.mqttmessage(
            req_topic.format(requestid=2), b'querytwo')
        ret = client_topics['linestart'].request(client, b'querytwo',
                                                 timeout=5,
                                                 archi='m3', num='1')
        server_cb.assert_called_with(request_msg, archi='m3', num='1')
        server_cb.reset_mock()
        server_cb.side_effect = None
        self.assertEqual(ret, b'responsetwo')

        # Timeout from the server
        server_cb.return_value = None  # ignore return value here
        request_msg = mqttclient_mock.mqttmessage(
            req_topic.format(requestid=3), b'querythree')

        self.assertRaises(RuntimeError,
                          client_topics['linestart'].request,
                          client, b'querythree', timeout=5,
                          archi='m3', num='1')
        server_cb.assert_called_with(request_msg, archi='m3', num='1')
        server_cb.reset_mock()

        # Publish timeout
        server_cb.return_value = b''
        client.publish_delay = 5
        server.publish_delay = 0
        request_msg = mqttclient_mock.mqttmessage(
            req_topic.format(requestid=4), b'queryfour')
        self.assertRaises(RuntimeError,
                          client_topics['linestart'].request,
                          client, b'queryfour', timeout=0.1,
                          archi='m3', num='1')
        self.assertFalse(server_cb.called)

        # Cleanup
        # Wait until callback called
        self.assertEqualTimeout(lambda: server_cb.called, True, timeout=10)


class InputOutputTest(AgentTest):
    """Test InputOutput classes."""

    def test_input_topics(self):
        """Test Input Topics."""
        topicname = '{procid}/fd/stdin'

        server_cb = mock.Mock()
        client_topics = {'in': mqttcommon.InputClient(topicname)}
        server_topics = {'in': mqttcommon.InputServer(topicname,
                                                      wrap_mock(server_cb))}

        client = mqttclient_mock.MQTTClientMock('localhost', 1883,
                                                list(client_topics.values()))
        mqttclient_mock.MQTTClientMock('localhost', 1883,
                                       list(server_topics.values()))

        # Client writes a message
        client_topics['in'].send(client, b'input',
                                 procid='1').wait_for_publish()
        self.assertTrue(server_cb.called)

        write_topic = server_topics['in'].topic.format(procid='1')
        write_msg = mqttclient_mock.mqttmessage(write_topic, b'input')
        server_cb.assert_called_with(write_msg, procid='1')

    def test_output_topics(self):
        """Test Output Topics."""
        topicname = '{procid}/fd/stdout'

        client_cb = mock.Mock()
        client_topics = {'out': mqttcommon.OutputClient(topicname,
                                                        wrap_mock(client_cb))}
        server_topics = {'out': mqttcommon.OutputServer(topicname)}

        client = mqttclient_mock.MQTTClientMock('localhost', 1883,
                                                list(client_topics.values()))
        mqttclient_mock.MQTTClientMock('localhost', 1883,
                                       list(server_topics.values()))

        # Server writes a message
        server_topics['out'].send(client, b'output',
                                  procid='1').wait_for_publish()
        self.assertTrue(client_cb.called)

        read_topic = client_topics['out'].topic.format(procid='1')
        read_msg = mqttclient_mock.mqttmessage(read_topic, b'output')
        client_cb.assert_called_with(read_msg, procid='1')


class ChannelTest(AgentTest):
    """Test Channel classes."""

    def test_channel_topic(self):
        """Test ChannelTopic."""
        topicname = '{archi}/{num}/line'
        output_topic = '{archi}/{num}/line/data/out'
        input_topic = '{archi}/{num}/line/data/in'

        # Verify topics values
        self.assertEqual(mqttcommon.ChannelTopic.output_topic(topicname),
                         output_topic)
        self.assertEqual(mqttcommon.ChannelTopic.input_topic(topicname),
                         input_topic)

        # Output/input conversion
        self.assertEqual(
            mqttcommon.ChannelTopic.output_topic_from_input(input_topic),
            output_topic)
        self.assertEqual(
            mqttcommon.ChannelTopic.input_topic_from_output(output_topic),
            input_topic)

        # Conversion is stable
        self.assertEqual(
            input_topic,
            mqttcommon.ChannelTopic.input_topic_from_output(
                mqttcommon.ChannelTopic.output_topic_from_input(input_topic)))
        self.assertEqual(
            output_topic,
            mqttcommon.ChannelTopic.output_topic_from_input(
                mqttcommon.ChannelTopic.input_topic_from_output(output_topic)))

    def test_channel_topics(self):
        """Test Channel Topics."""
        topicname = '{archi}/{num}/line'

        client_cb = mock.Mock()
        server_cb = mock.Mock()
        client_topics = {'line': mqttcommon.ChannelClient(
            topicname, wrap_mock(client_cb))}
        server_topics = {'line': mqttcommon.ChannelServer(
            topicname, wrap_mock(server_cb))}

        client = mqttclient_mock.MQTTClientMock('localhost', 1883,
                                                list(client_topics.values()))
        server = mqttclient_mock.MQTTClientMock('localhost', 1883,
                                                list(server_topics.values()))

        # Client writes a message
        client_topics['line'].send(client, b'req',
                                   archi='m3', num=1).wait_for_publish()
        self.assertTrue(server_cb.called)

        write_topic = server_topics['line'].topic.format(archi='m3', num='1')
        write_msg = mqttclient_mock.mqttmessage(write_topic, b'req')
        server_cb.assert_called_with(write_msg, archi='m3', num='1')

        # Server writes a message
        line_write = server_topics['line'].output_publisher(
            server, archi='m3', num='1')
        line_write(b'response').wait_for_publish()
        self.assertTrue(client_cb.called)

        read_topic = client_topics['line'].topic.format(archi='m3', num='1')
        read_msg = mqttclient_mock.mqttmessage(read_topic, b'response')
        client_cb.assert_called_with(read_msg, archi='m3', num='1')


class ErrorTest(AgentTest):
    """Test Error classes."""

    def test_error_topic(self):
        """Test ErrorTopic."""
        topic = 'iot-lab/serial/grenoble'
        error_topic = 'iot-lab/serial/grenoble/error/'
        subscribe_topic = 'iot-lab/serial/grenoble/error/#'

        relative_topic_none = ''
        relative_topic_example = 'm3/1/line/data/in'

        # Verify topics values
        self.assertEqual(mqttcommon.ErrorTopic.error_topic(topic), error_topic)
        self.assertEqual(mqttcommon.ErrorTopic.subscribe_topic(topic),
                         subscribe_topic)

        # Test relative topic stability
        self.assertEqual(
            relative_topic_none,
            mqttcommon.ErrorTopic.relative_topic(
                topic, os.path.join(topic, relative_topic_none)))

        self.assertEqual(
            relative_topic_example,
            mqttcommon.ErrorTopic.relative_topic(
                topic, os.path.join(topic, relative_topic_example)))

        self.assertRaises(ValueError, mqttcommon.ErrorTopic.relative_topic,
                          topic, 'topic/not/starting/with/topic')

    def test_error_topics(self):
        """Test Error Topics."""
        topicname = 'iot-lab/serial/grenoble'
        error_topic = 'iot-lab/serial/grenoble/error/'

        client_cb = mock.Mock()
        client_topics = {'error': mqttcommon.ErrorClient(
            topicname, wrap_mock(client_cb))}
        server_topics = {'error': mqttcommon.ErrorServer(topicname)}

        mqttclient_mock.MQTTClientMock('localhost', 1883,
                                       list(client_topics.values()))
        server = mqttclient_mock.MQTTClientMock('localhost', 1883,
                                                list(server_topics.values()))
        server.publish_delay = 0

        err = b'error message'
        topic_with_error_rel = 'm3/1/line/data/in'
        topic_with_error = os.path.join(topicname, topic_with_error_rel)
        server_topics['error'].publish_error(server, topic_with_error, err)

        self.assertTrue(client_cb.called)

        err_msg = mqttclient_mock.mqttmessage(
            os.path.join(error_topic, topic_with_error_rel),
            err)
        client_cb.assert_called_with(err_msg, topic_with_error_rel)



# -*- coding:utf-8 -*-
"""Integration tests for IoT-LAB MQTT process client and server."""

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import time
import contextlib

import mock

from iotlabmqtt import process
from iotlabmqtt.clients import process as process_client

from . import IntegrationTestCase
from .. import TestCaseImproved

# pylint:disable=invalid-name

PROC_END_FMT = (
    "stderr({procid}): ''\n"
    "(Cmd) stdout({procid}): ''\n"
    "(Cmd) closed({procid}): {ret}\n"
    "(Cmd) "
)


class ProcessIntegrationTests(IntegrationTestCase):
    """Test process client and server using a broker."""

    @contextlib.contextmanager
    def start_client_and_server(self, brokerport):
        """Start process client and server context manager.

        Yields client and stdout mock.
        """
        # pylint:disable=attribute-defined-outside-init
        args = ['localhost', '--broker-port', '%s' % brokerport,
                '--prefix', 'process/test/prefix']
        opts = process.PARSER.parse_args(args)
        server = process.MQTTProcessAgent.from_opts_dict(**vars(opts))
        server.start()
        self.server = server
        try:
            args = ['localhost', '--broker-port', '%s' % brokerport,
                    '--prefix', 'process/test/prefix',
                    '--site', server.HOSTNAME]
            opts = process_client.PARSER.parse_args(args)
            client = process_client.ProcessShell.from_opts_dict(**vars(opts))
            client.start()

            try:
                with mock.patch('sys.stdout', new_callable=StringIO) as stdout:
                    yield client, stdout
            finally:
                client.stop()
        finally:
            server.stop()

    def test_process_agent_new_free_list(self):
        """Test process agent new, free and list."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_new_free_list(client, stdout)

    def _test_process_agent_new_free_list(self, client, stdout):
        """Test process agent new, free and list.

        Play with allocation and release.
        """
        # pylint:disable=too-many-statements

        client.onecmd('list')
        self.assertEqual(stdout.getvalue(), '[]\n')
        stdout.seek(0)
        stdout.truncate(0)

        # Create two processes
        client.onecmd('new')
        self.assertEqual(stdout.getvalue(), '1\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('new procname')
        self.assertEqual(stdout.getvalue(), 'procname\n')
        stdout.seek(0)
        stdout.truncate(0)

        # Already used
        client.onecmd('new procname')
        self.assertEqual(stdout.getvalue(), 'name procname already in use\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('list')
        self.assertEqual(stdout.getvalue(), "['1', 'procname']\n")
        stdout.seek(0)
        stdout.truncate(0)

        # Two free on same name
        client.onecmd('free 1')
        self.assertEqual(stdout.getvalue(), '')

        client.onecmd('free 1')
        self.assertEqual(stdout.getvalue(), 'Name 1 not found\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('list')
        self.assertEqual(stdout.getvalue(), "['procname']\n")
        stdout.seek(0)
        stdout.truncate(0)

        # Create manually '2'
        # Then new will create 3 as 2 already exists
        client.onecmd('new 2')
        self.assertEqual(stdout.getvalue(), '2\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('new')
        self.assertEqual(stdout.getvalue(), '3\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('list')
        self.assertEqual(stdout.getvalue(), "['2', '3', 'procname']\n")
        stdout.seek(0)
        stdout.truncate(0)

        # Free all
        client.onecmd('free 3')
        self.assertEqual(stdout.getvalue(), '')

        client.onecmd('free 2')
        self.assertEqual(stdout.getvalue(), '')

        client.onecmd('free procname')
        self.assertEqual(stdout.getvalue(), '')

        client.onecmd('list')
        self.assertEqual(stdout.getvalue(), '[]\n')
        stdout.seek(0)
        stdout.truncate(0)

    def test_process_agent_run_stdout(self):
        """Test process agent running with stdout."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_run_stdout(client, stdout)

    def _test_process_agent_run_stdout(self, client, stdout):
        """Test process agent running with stdout."""
        stdout_file = self.file_path('print_stdout.sh')
        # spaces not supported
        self.assertTrue(' ' not in stdout_file)

        # Sanity check
        client.onecmd('list')
        self.assertEqual(stdout.getvalue(), '[]\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('new')
        self.assertEqual(stdout.getvalue(), '1\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('list')
        self.assertEqual(stdout.getvalue(), "['1']\n")
        stdout.seek(0)
        stdout.truncate(0)

        # No active process on 1
        client.onecmd('stdinline 1 message')
        err = ('PROCESS ERROR: process/1: Process not running\n'
               '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, err, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # scripts runs for 2 seconds
        client.onecmd('run 1 %s %s %s' % (stdout_file, '20', '0.1'))
        messages = (
            "stdout(1): 'Message 1\\n'\n"
            "(Cmd) stdout(1): 'Message 2\\n'\n"
            "(Cmd) stdout(1): 'Message 3\\n'\n"
            "(Cmd) stdout(1): 'Message 4\\n'\n"
            "(Cmd) stdout(1): 'Message 5\\n'\n"
            "(Cmd) stdout(1): 'Message 6\\n'\n"
            "(Cmd) stdout(1): 'Message 7\\n'\n"
            "(Cmd) stdout(1): 'Message 8\\n'\n"
            "(Cmd) stdout(1): 'Message 9\\n'\n"
            "(Cmd) stdout(1): 'Message 10\\n'\n"
            "(Cmd) stdout(1): 'Message 11\\n'\n"
            "(Cmd) stdout(1): 'Message 12\\n'\n"
            "(Cmd) stdout(1): 'Message 13\\n'\n"
            "(Cmd) stdout(1): 'Message 14\\n'\n"
            "(Cmd) stdout(1): 'Message 15\\n'\n"
            "(Cmd) stdout(1): 'Message 16\\n'\n"
            "(Cmd) stdout(1): 'Message 17\\n'\n"
            "(Cmd) stdout(1): 'Message 18\\n'\n"
            "(Cmd) stdout(1): 'Message 19\\n'\n"
            "(Cmd) stdout(1): 'Message 20\\n'\n"
            "(Cmd) "
        ) + PROC_END_FMT.format(procid='1', ret='0')

        # Output may not always be ordered the same for stderr, compare sorted
        self.assertEqualTimeout(
            lambda: sorted(stdout.getvalue().splitlines(True)),
            sorted(messages.splitlines(True)),
            10)
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('poll 1')
        self.assertEqual(stdout.getvalue(), '0\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('kill 1')
        self.assertEqual(stdout.getvalue(), '')

        client.onecmd('poll 1')
        self.assertEqual(stdout.getvalue(), '0\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('free 1')
        self.assertEqual(stdout.getvalue(), '')

        client.onecmd('list')
        self.assertEqual(stdout.getvalue(), "[]\n")
        stdout.seek(0)
        stdout.truncate(0)

    def test_process_agent_run_stdin_and_stderr(self):
        """Test process agent running with sdin and stderr."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_run_stdin_and_stderr(client, stdout)

    def _test_process_agent_run_stdin_and_stderr(self, client, stdout):
        """Test process agent running with stdin and stderr."""
        stderr_file = self.file_path('echo_stderr.sh')
        # spaces not supported
        self.assertTrue(' ' not in stderr_file)

        client.onecmd('new echo')
        self.assertEqual(stdout.getvalue(), 'echo\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('run echo %s' % stderr_file)
        self.assertEqual(stdout.getvalue(), '')

        # Send messages to stdin one by one
        # first
        client.onecmd('stdinline echo first')
        messages = (
            "stderr(echo): 'first\\n'\n"
            "(Cmd) "
        )
        self.assertEqualTimeout(stdout.getvalue, messages, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # second
        client.onecmd('stdinraw echo se')
        client.onecmd('stdinraw echo co')
        client.onecmd('stdinline echo nd')
        messages = (
            "stderr(echo): 'second\\n'\n"
            "(Cmd) "
        )
        self.assertEqualTimeout(stdout.getvalue, messages, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # third
        client.onecmd('stdinline echo third')
        messages = (
            "stderr(echo): 'third\\n'\n"
            "(Cmd) "
        )
        self.assertEqualTimeout(stdout.getvalue, messages, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # Now check process running and stop it
        client.onecmd('poll echo')
        self.assertEqual(stdout.getvalue(), '')

        stdout.write('(Cmd) ')  # Add (Cmd) to help compare order
        client.onecmd('kill echo')
        # First one could be different (stdout or stderr) and this changes
        # which one has '(Cmd) '. So compare without it.
        # Output without '(Cmd) '
        output = '(Cmd) ' + PROC_END_FMT.format(procid='echo', ret='-15')
        self.assertEqualTimeout(
            lambda: sorted(stdout.getvalue().splitlines(True)),
            sorted(output.splitlines(True)), 10)
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('poll echo')
        self.assertEqual(stdout.getvalue(), '-15\n')
        stdout.seek(0)
        stdout.truncate(0)

        # Ignore messages when stopped
        client.onecmd('stdinline echo ignored message')
        time.sleep(2)
        self.assertEqual(stdout.getvalue(), '')

    def test_process_agent_dynamic_behaviour(self):
        """Test process agent running dynamic behaviour."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_dynamic_behaviour(client, stdout)

    def _test_process_agent_dynamic_behaviour(self, client, stdout):
        """Test process agent running dynamic behaviour."""
        stderr_file = self.file_path('echo_stderr.sh')
        # spaces not supported
        self.assertTrue(' ' not in stderr_file)

        client.onecmd('new 2')
        self.assertEqual(stdout.getvalue(), '2\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('run 2 %s' % stderr_file)
        self.assertEqual(stdout.getvalue(), '')

        # Invalid commands while running
        client.onecmd('free 2')
        err = 'Process still running wait or kill it first\n'
        self.assertEqual(stdout.getvalue(), err)
        stdout.seek(0)
        stdout.truncate(0)

        err_stop_first = 'Process still running, be stopped before running\n'
        client.onecmd('rerun 2')
        self.assertEqual(stdout.getvalue(), err_stop_first)
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('run 2 anything')
        self.assertEqual(stdout.getvalue(), err_stop_first)
        stdout.seek(0)
        stdout.truncate(0)

        stdout.write('(Cmd) ')  # Add (Cmd) to help compare order
        client.onecmd('kill 2')
        output = '(Cmd) ' + PROC_END_FMT.format(procid='2', ret='-15')
        self.assertEqualTimeout(
            lambda: sorted(stdout.getvalue().splitlines(True)),
            sorted(output.splitlines(True)), 10)
        stdout.seek(0)
        stdout.truncate(0)

        # Try reruning
        client.onecmd('rerun 2')
        self.assertEqual(stdout.getvalue(), '')

        client.onecmd('poll 2')
        self.assertEqual(stdout.getvalue(), '')

        stdout.write('(Cmd) ')  # Add (Cmd) to help compare order
        client.onecmd('kill 2')
        output = '(Cmd) ' + PROC_END_FMT.format(procid='2', ret='-15')
        self.assertEqualTimeout(
            lambda: sorted(stdout.getvalue().splitlines(True)),
            sorted(output.splitlines(True)), 10)
        stdout.seek(0)
        stdout.truncate(0)

    def test_process_agent_python_buffered(self):
        """Test process agent running with a python program."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_python_buffered(client, stdout)

    def _test_process_agent_python_buffered(self, client, stdout):
        """Test process agent running with a python program."""
        script_file = self.file_path('print_one_line.py')
        # spaces not supported
        self.assertTrue(' ' not in script_file)

        client.onecmd('new python')
        self.assertEqual(stdout.getvalue(), 'python\n')
        stdout.seek(0)
        stdout.truncate(0)

        # Message is received because python is run with PYTHONUNBUFFERED
        client.onecmd('run python %s' % script_file)
        messages = ("stdout(python): 'Message\\n'\n"
                    "(Cmd) ")
        self.assertEqualTimeout(stdout.getvalue, messages, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # kill
        stdout.write('(Cmd) ')  # Add (Cmd) to help compare order
        client.onecmd('kill python')
        output = '(Cmd) ' + PROC_END_FMT.format(procid='python', ret='-15')
        self.assertEqualTimeout(
            lambda: sorted(stdout.getvalue().splitlines(True)),
            sorted(output.splitlines(True)), 10)
        stdout.seek(0)
        stdout.truncate(0)

    def test_process_agent_kill(self):
        """Test process agent kill cases."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_kill(client, stdout)

    def _test_process_agent_kill(self, client, stdout):
        """Test process agent kill cases."""
        trap_sigterm_file = self.file_path('trap_sigterm.sh')

        client.onecmd('new')
        self.assertEqual(stdout.getvalue(), '1\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('run 1 %s' % trap_sigterm_file)
        self.assertEqual(stdout.getvalue(), '')

        # Kill process, will be killed by SIGKILL as return value is "-9"
        stdout.write('(Cmd) ')  # Add (Cmd) to help compare order
        client.onecmd('kill 1')
        output = '(Cmd) ' + (
            "stdout(1): 'Process caught SIGTERM\\n'\n"
            "(Cmd) "
        ) + PROC_END_FMT.format(procid='1', ret='-9')
        self.assertEqualTimeout(
            lambda: sorted(stdout.getvalue().splitlines(True)),
            sorted(output.splitlines(True)), 10)
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('poll 1')
        self.assertEqual(stdout.getvalue(), '-9\n')
        stdout.seek(0)
        stdout.truncate(0)

        # Re-kill
        client.onecmd('kill 1')
        self.assertEqual(stdout.getvalue(), '')

        client.onecmd('poll 1')
        self.assertEqual(stdout.getvalue(), '-9\n')
        stdout.seek(0)
        stdout.truncate(0)

    def test_process_agent_stop_kill(self):
        """Test process agent stop that kills running processes."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_stop_kill(client, stdout)

    def _test_process_agent_stop_kill(self, client, stdout):
        """Test process agent stop that kills running processes.

        Server is stopped before client to still get messages on it.
        """
        trap_sigterm_file = self.file_path('trap_sigterm.sh')

        client.onecmd('new stop')
        self.assertEqual(stdout.getvalue(), 'stop\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('run stop %s' % trap_sigterm_file)
        self.assertEqual(stdout.getvalue(), '')

        stdout.write('(Cmd) ')  # Add (Cmd) to help compare order
        # Stop server, program should be killed by this
        self.server.stop()

        # Check process has indeed been killed
        output = '(Cmd) ' + (
            "stdout(stop): 'Process caught SIGTERM\\n'\n"
            "(Cmd) "
        ) + PROC_END_FMT.format(procid='stop', ret='-9')
        self.assertEqualTimeout(
            lambda: sorted(stdout.getvalue().splitlines(True)),
            sorted(output.splitlines(True)), 10)
        stdout.seek(0)
        stdout.truncate(0)

    def test_process_agent_not_allocated(self):
        """Test process agent when not allocated."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_not_allocated(client, stdout)

    def _test_process_agent_not_allocated(self, client, stdout):
        """Test process agent when not allocated."""

        # Non existant
        client.onecmd('poll 1')
        self.assertEqual(stdout.getvalue(), 'Name 1 not found\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('kill 1')
        self.assertEqual(stdout.getvalue(), 'Name 1 not found\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('run 1 comand')
        self.assertEqual(stdout.getvalue(), 'Name 1 not found\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('rerun 1')
        self.assertEqual(stdout.getvalue(), 'Name 1 not found\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('stdinline 1 message')
        err = ('PROCESS ERROR: process/1/fd/stdin: No process on 1\n'
               '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, err, 2)
        stdout.seek(0)
        stdout.truncate(0)

    def test_process_agent_invalid_cases(self):
        """Test process agent running invalid cases."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_process_agent_invalid_cases(client, stdout)

    def _test_process_agent_invalid_cases(self, client, stdout):
        """Test process agent running invalid cases."""

        invalid_bytes = b'this: \xc3\x28 is invalid\n'

        # Invalid characters
        err_invalid = 'Name should not contain %s\n' % '/{}+#'
        for name in ('in/valid', 'in{val}id', 'inval+d', 'inval#d'):
            client.onecmd('new %s' % name)
            self.assertEqual(stdout.getvalue(), err_invalid)
            stdout.seek(0)
            stdout.truncate(0)

        # Invalid unicode payload
        ret = client.topics['new'].request(client.client, invalid_bytes,
                                           timeout=10, procid='1')
        self.assertEqual(ret.decode('utf-8'), 'Error decoding utf-8 payload')

        ret = client.topics['free'].request(client.client, invalid_bytes,
                                            timeout=10, procid='1')
        self.assertEqual(ret.decode('utf-8'), 'Error decoding utf-8 payload')

        # Command on non run ones
        client.onecmd('new')
        self.assertEqual(stdout.getvalue(), '1\n')
        stdout.seek(0)
        stdout.truncate(0)

        # Invalid unicode payload
        ret = client.topics['procrun'].request(client.client, invalid_bytes,
                                               timeout=10, procid='1')
        self.assertEqual(ret.decode('utf-8'), 'Error decoding utf-8 payload')

        # Invalid json
        payload = '{"a":'.encode('utf-8')
        ret = client.topics['procrun'].request(client.client, payload,
                                               timeout=10, procid='1')
        self.assertEqual(ret.decode('utf-8'),
                         'Invalid command, should be a json encoded list')

        # Invalid json value
        payload = '3'.encode('utf-8')
        ret = client.topics['procrun'].request(client.client, payload,
                                               timeout=10, procid='1')
        self.assertEqual(ret.decode('utf-8'),
                         'Invalid command')

        client.onecmd('poll 1')
        self.assertEqual(stdout.getvalue(), 'Error process never run\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('rerun 1')
        self.assertEqual(stdout.getvalue(), 'Error process never run\n')
        stdout.seek(0)
        stdout.truncate(0)

        client.onecmd('run 1 invalid_command/no_path')
        # Python3 adds file path
        err = '[Errno 2] No such file or directory'
        self.assertTrue(stdout.getvalue().startswith(err))
        stdout.seek(0)
        stdout.truncate(0)


@mock.patch('sys.stdout', new_callable=StringIO)
class ProcessClientErrorTests(TestCaseImproved):
    """Test ProcessClient Parsing errors."""
    def setUp(self):
        args = ['localhost', '--broker-port', '%s' % 54321,
                '--prefix', 'process/test/prefix',
                '--site', process.MQTTProcessAgent.HOSTNAME]
        opts = process_client.PARSER.parse_args(args)
        self.client = process_client.ProcessShell.from_opts_dict(**vars(opts))
        # Publish should crash
        self.client.publish = None

    def test_new(self, stdout):
        """Test new parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: new [NAME]\n'
               '  NAME: optional process identifier\n')

        # Too many arguments
        self.client.onecmd('new name additional')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_free(self, stdout):
        """Test free parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: free NAME\n'
               '  NAME: process identifier\n')

        # Not enough arguments
        self.client.onecmd('free')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Too many arguments
        self.client.onecmd('free name additional')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_list(self, stdout):
        """Test list help."""
        hlp = 'list\n'

        # cannot be done from command, just call it
        self.client.onecmd('help list')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_run(self, stdout):
        """Test run parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: run NAME CMD0 [CMD1 [...]]\n'
               '  NAME: process identifier\n'
               '  CMD: command arguments\n')

        # Not enough arguments
        self.client.onecmd('run')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        self.client.onecmd('run procid1')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_poll(self, stdout):
        """Test poll parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: poll NAME\n'
               '  NAME: process identifier\n')

        # Not enough arguments
        self.client.onecmd('poll')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Too many arguments
        self.client.onecmd('poll name additional')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_kill(self, stdout):
        """Test kill parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: kill NAME\n'
               '  NAME: process identifier\n')

        # Not enough arguments
        self.client.onecmd('kill')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Too many arguments
        self.client.onecmd('kill name additional')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_rerun(self, stdout):
        """Test rerun parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: rerun NAME\n'
               '  NAME: process identifier\n')

        # Not enough arguments
        self.client.onecmd('rerun')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Too many arguments
        self.client.onecmd('rerun name additional')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_stdinline(self, stdout):
        """Test stdinline parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: stdinline NAME message with spaces\n'
               '  NAME: process identifier\n'
               '  Message is sent as utf-8 data with a \\n character\n')

        # Not enough arguments
        self.client.onecmd('stdinline')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        self.client.onecmd('stdinline name')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_stdinraw(self, stdout):
        """Test stdinraw parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: stdinraw NAME message with spaces\n'
               '  NAME: process identifier\n'
               '  Message is sent as utf-8 data\n')

        # Not enough arguments
        self.client.onecmd('stdinraw')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        self.client.onecmd('stdinraw name')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)



# -*- coding:utf-8 -*-
"""Integration tests for IoT-LAB MQTT radiosniffer client and server."""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from builtins import *  # pylint:disable=W0401,W0614,W0622

import os
import time
import contextlib
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import json
import mock

from iotlabmqtt import radiosniffer
from iotlabmqtt.clients import radiosniffer as radiosniffer_client

from . import IntegrationTestCase
from .. import TestCaseImproved

# pylint:disable=invalid-name


def generate_zep_packet():
    """Generate a zep packet."""

    import binascii
    zep_msg = (
        '45 58 02 01'   # Base Zep header
        '0B 00 01 00 ff'   # chan | dev_id | dev_id| LQI/CRC_MODE |  LQI

        '83 aa 7e 80'   # Timestamp msb (Epoch 1/1/1970)
        '00 00 00 00'   # timestamp lsp

        '00 00 00 01'   # seqno

        '00 01 02 03'   # reserved 0-3/10
        '04 05 16 07'   # reserved 4-7/10
        '08 09'         # reserved 8-9 / 10
        '08'            # Length 2 + data_len
        '61 62 63'      # Data
        '41 42 43'      # Data
        'FF FF'         # CRC)
    )
    zep_msg = zep_msg.replace(' ', '')
    return binascii.a2b_hex(zep_msg)


class RadioSnifferIntegrationTest(IntegrationTestCase):
    """Test sniffer client and server using a broker."""

    def setUp(self):
        self.socat = {}
        for port in range(30001, 30003):
            self.socat_start(port)

    def tearDown(self):
        for port in list(self.socat):
            self.socat_stop(port)

    @contextlib.contextmanager
    def start_client_and_server(self, brokerport):
        """Start radiosiffer client and server context manager.

        Yields client and stdout mock.
        """
        # pylint:disable=attribute-defined-outside-init
        args = ['localhost', '--broker-port', '%s' % brokerport,
                '--prefix', 'radiosniffer/test/prefix']
        args += ['--experiment-id', '12345',
                 '--iotlab-user', 'us3rn4me',
                 '--iotlab-password', 'p4sswd']
        opts = radiosniffer.PARSER.parse_args(args)

        with mock.patch('iotlabcli.experiment.get_experiment') as get_exp:
            get_exp.return_value = {'state': 'Running'}

            server = radiosniffer.MQTTRadioSnifferAggregator.from_opts_dict(
                **vars(opts))

        assert server.iotlabapi.expid == 12345
        assert server.iotlabapi.api.auth.username == 'us3rn4me'
        assert server.iotlabapi.api.auth.password == 'p4sswd'
        server.iotlabapi.api = None  # Breaks on calls

        server.start()
        self.server = server

        try:
            args = ['localhost', '--broker-port', '%s' % brokerport,
                    '--prefix', 'radiosniffer/test/prefix',
                    '--site', server.HOSTNAME]
            opts = radiosniffer_client.PARSER.parse_args(args)
            client = radiosniffer_client.RadioSnifferShell.from_opts_dict(
                **vars(opts))
            client.start()

            try:
                with mock.patch('sys.stdout', new_callable=StringIO) as stdout:
                    yield client, stdout
            finally:
                client.stop()
        finally:
            server.stop()

    def test_radiosniffer_agent_one_node(self):
        """Test radiosniffer agent normal case."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_radiosniffer_agent_one_node(client, stdout)

    def _test_radiosniffer_agent_one_node(self, client, stdout):
        """Simple check one node sniffer.

        Global header, start, 4 packets, stop, close pcap file.
        """
        self.server.iotlabapi.set_sniffer_channel = mock.Mock()

        port = 30001
        soc = self.socat[port]
        channel = 11
        outpcap = '/tmp/test.pcap'

        # Init Global pcap
        client.onecmd('rawpcap %s' % outpcap)
        out = 'Writing RAW PCAP header: 24 bytes\n'
        self.assertEqual(stdout.getvalue(), out)
        stdout.seek(0)
        stdout.truncate(0)

        # Start sniffer raw
        self.server.iotlabapi.set_sniffer_channel.return_value = {'30001': ''}

        client.onecmd('rawstart localhost %u %u' % (port, channel))
        self.server.iotlabapi.set_sniffer_channel.assert_called_with(
            channel, 'localhost', '30001')
        self.assertEqual(stdout.getvalue(), '')

        # Send 4 packets
        soc.stdin.write(generate_zep_packet())
        soc.stdin.write(generate_zep_packet())
        soc.stdin.write(generate_zep_packet())
        soc.stdin.write(generate_zep_packet())
        soc.stdin.flush()

        answer = ('raw_handler(localhost-30001): PKT: len(24)\n'
                  '(Cmd) raw_handler(localhost-30001): PKT: len(24)\n'
                  '(Cmd) raw_handler(localhost-30001): PKT: len(24)\n'
                  '(Cmd) raw_handler(localhost-30001): PKT: len(24)\n'
                  '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, answer, 5)
        stdout.seek(0)
        stdout.truncate(0)

        # Stop when running
        client.onecmd('stop localhost %u' % port)
        self.assertEqual(stdout.getvalue(), '')

        # Close packet
        client.onecmd('rawpcapclose')
        self.assertEqual(stdout.getvalue(), '')

        # outfile only has global header in it
        self.assertEqual(os.stat(outpcap).st_size, 24 + 4 * 24)

    def test_radiosniffer_agent_one_node_complex(self):
        """Test radiosniffer agent complex cases."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_radiosniffer_agent_one_node_complex(client, stdout)

    def _test_radiosniffer_agent_one_node_complex(self, client, stdout):
        """Wrong order start/stop sending messages without an output file."""
        self.server.iotlabapi.set_sniffer_channel = mock.Mock()

        port = 30001
        soc = self.socat[port]
        channel = 11
        outpcap = '/tmp/test.pcap'

        # Stop for nothing
        client.onecmd('stop localhost %u' % port)
        self.assertEqual(stdout.getvalue(), '')

        # Stopall for nothing
        client.onecmd('stopall')
        self.assertEqual(stdout.getvalue(), '')

        # Init Global pcap
        client.onecmd('rawpcap %s' % outpcap)
        out = 'Writing RAW PCAP header: 24 bytes\n'
        self.assertEqual(stdout.getvalue(), out)
        stdout.seek(0)
        stdout.truncate(0)

        # Close packet
        client.onecmd('rawpcapclose')
        self.assertEqual(stdout.getvalue(), '')

        # Start sniffer raw
        self.server.iotlabapi.set_sniffer_channel.return_value = {'30001': ''}
        client.onecmd('rawstart localhost %u %u' % (port, channel))
        self.server.iotlabapi.set_sniffer_channel.assert_called_with(
            channel, 'localhost', '30001')
        self.assertEqual(stdout.getvalue(), '')

        # Start sniffer raw again
        client.onecmd('rawstart localhost %u %u' % (port, channel))
        err = 'Already started, stop before start to change channel\n'
        self.assertEqual(stdout.getvalue(), err)
        stdout.seek(0)
        stdout.truncate(0)

        # Send 3 packets
        # First packet split before full length to execute the code handling
        # this
        pkt = generate_zep_packet()
        soc.stdin.write(pkt[:-2])
        soc.stdin.flush()
        time.sleep(1)  # wait for the packet to be handled
        soc.stdin.write(pkt[-2:])
        soc.stdin.flush()

        # Two packets full
        soc.stdin.write(generate_zep_packet())
        soc.stdin.write(generate_zep_packet())
        soc.stdin.flush()

        answer = ('raw_handler(localhost-30001): PKT: len(24)\n'
                  '(Cmd) raw_handler(localhost-30001): PKT: len(24)\n'
                  '(Cmd) raw_handler(localhost-30001): PKT: len(24)\n'
                  '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, answer, 5)
        stdout.seek(0)
        stdout.truncate(0)

        # Stop when running
        client.onecmd('stop localhost %u' % port)
        self.assertEqual(stdout.getvalue(), '')

        # outfile only has global header in it
        self.assertEqual(os.stat(outpcap).st_size, 24)

    def test_radiosniffer_agent_invalid_cases(self):
        """Test radiosniffer agent invalid cases."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_radiosniffer_agent_invalid_cases(client, stdout)

    def _test_radiosniffer_agent_invalid_cases(self, client, stdout):
        port = 30001

        # Start sniffer raw invalid channel
        client.onecmd('rawstart localhost %u %u' % (port, 42))
        error = 'Invalid channel value, should be in [11, 26]\n'
        self.assertEqual(stdout.getvalue(), error)
        stdout.seek(0)
        stdout.truncate(0)

    def test_radiosniffer_agent_sniffer_fail(self):
        """Test radiosniffer agent sniffer set fail."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_radiosniffer_agent_sniffer_fail(client, stdout)

    def _test_radiosniffer_agent_sniffer_fail(self, client, stdout):
        """Check setting sniffer but fails."""
        self.server.iotlabapi.set_sniffer_channel = mock.Mock()

        # Start sniffer raw
        self.server.iotlabapi.set_sniffer_channel.return_value = {
            '30001': 'Update profile failed: error string'}

        client.onecmd('rawstart localhost %u %u' % (30001, 11))
        self.server.iotlabapi.set_sniffer_channel.assert_called_with(
            11, 'localhost', '30001')
        err = 'Update profile failed: error string\n'
        self.assertEqual(stdout.getvalue(), err)
        stdout.seek(0)
        stdout.truncate(0)

        # Start sniffer but connection fails
        assert 30000 not in self.socat, 'port 30000 is used'
        self.server.iotlabapi.set_sniffer_channel.return_value = {'30000': ''}
        client.onecmd('rawstart localhost %u %u' % (30000, 26))
        self.server.iotlabapi.set_sniffer_channel.assert_called_with(
            26, 'localhost', '30000')
        err = 'Connection failed: [Errno 111] Connection refused\n'
        self.assertEqual(stdout.getvalue(), err)
        stdout.seek(0)
        stdout.truncate(0)

    def test_radiosniffer_agent_sniffer_disconnect(self):
        """Test radiosniffer agent sniffer disconnection."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_radiosniffer_agent_sniffer_disconnect(client, stdout)

    def _test_radiosniffer_agent_sniffer_disconnect(self, client, stdout):
        """Check setting sniffer but fails."""
        # Don't mock set_sniffer_channel, should work for localhost
        port = 30001
        channel = 11

        # Start sniffer and connection breaks
        client.onecmd('rawstart localhost %u %u' % (port, channel))
        self.assertEqual(stdout.getvalue(), '')

        # Stop conn
        time.sleep(1)
        self.socat_stop(port)

        # Error raised
        err = ('RADIO SNIFFER ERROR: localhost/30001: '
               'Connection closed in state raw\n'
               '(Cmd) ')
        self.assertEqualTimeout(stdout.getvalue, err, 2)
        stdout.seek(0)
        stdout.truncate(0)

        # Restart socat
        self.socat_start(port)

    def test_radiosniffer_agent_sniffer_fail_m3_a8(self):
        """Test radiosniffer agent sniffer connection failed for m3/a8."""
        with self.start_client_and_server(self.BROKERPORT) as (client, stdout):
            self._test_radiosniffer_agent_sniffer_fail_m3_a8(client, stdout)

    @mock.patch('iotlabcli.node.node_command')
    def _test_radiosniffer_agent_sniffer_fail_m3_a8(self, client, stdout,
                                                    node_command=None):
        """Check setting sniffer but fails for connectiong m3/a8.

        Mock iotlabapi at low level to try the whole integration.
        I want to increase the chances for it to work
        """
        site = self.server.iotlabapi.site

        # M3-1234 channel 11

        node_host = 'm3-1234.%s.iot-lab.info' % site

        # pylint:disable=attribute-defined-outside-init
        self.cmd_opt_content = None

        def node_command_mock(api, command, exp_id, nodes, cmd_opt):
            """Mock that saves profile content."""
            # pylint:disable=unused-argument
            self.cmd_opt_content = open(cmd_opt).read()
            return mock.DEFAULT

        node_command.side_effect = node_command_mock
        node_command.return_value = {'0': [node_host]}

        # Start sniffer but connection fails
        client.onecmd('rawstart m3 1234 11')
        error_start = 'Connection failed: [Errno -'
        # -5] No address associated with hostname')
        # got a different message on ArchLinux
        # -2] Name or service not known
        self.assertTrue(stdout.getvalue().startswith(error_start))
        stdout.seek(0)
        stdout.truncate(0)

        # Verify add_profile call
        api, cmd, exp_id, nodes, _ = node_command.call_args[0]
        self.assertEqual(
            (api, cmd, exp_id, nodes),
            (self.server.iotlabapi.api, 'profile-load', 12345, [node_host]))

        profile_m3_str = (
            '{\n'
            '    "consumption": null, \n'
            '    "nodearch": "m3", \n'
            '    "power": "dc", \n'
            '    "profilename": "iotlabmqtt_11_m3", \n'
            '    "radio": {\n'
            '        "channels": [\n'
            '            11\n'
            '        ], \n'
            '        "mode": "sniffer", \n'
            '        "num_per_channel": null, \n'
            '        "period": null\n'
            '    }\n'
            '}'
        )
        # Compare loaded json, as python3 changed the dumped format
        self.assertEqual(json.loads(self.cmd_opt_content),
                         json.loads(profile_m3_str))

        # Test A8
        node_command.return_value = {
            '0': ['a8-1234.%s.iot-lab.info' % site],
        }
        # Start sniffer on a8 but connection fails
        client.onecmd('rawstart a8 1234 11')
        error_start = 'Connection failed: [Errno -'
        # -5] No address associated with hostname')
        # got a different message on ArchLinux
        # -2] Name or service not known
        self.assertTrue(stdout.getvalue().startswith(error_start))
        stdout.seek(0)
        stdout.truncate(0)


@mock.patch('sys.stdout', new_callable=StringIO)
class RadioSnifferClientErrorTests(TestCaseImproved):
    """Test RadioSniffer parsing errors."""
    def setUp(self):
        args = ['localhost', '--broker-port', '%s' % 54321,
                '--prefix', 'radiosniffer/test/prefix',
                '--site', radiosniffer.MQTTRadioSnifferAggregator.HOSTNAME]
        opts = radiosniffer_client.PARSER.parse_args(args)
        self.client = radiosniffer_client.RadioSnifferShell.from_opts_dict(
            **vars(opts))
        # Publish should crash
        self.client.publish = None

    def test_rawpcap(self, stdout):
        """Test rawpcap errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: rawpcap FILEPATH\n'
               '  FILEPATH: rawpcap output\n')

        # No file
        self.client.onecmd('rawpcap')
        err = "Could not open file: [Errno 2] No such file or directory: ''\n"
        self.assertEqual(stdout.getvalue(), err + hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Cannot open file
        self.client.onecmd('rawpcap /non_existing/dir/and/file')
        err = ("Could not open file: [Errno 2] No such file or directory: "
               "'/non_existing/dir/and/file'\n")
        self.assertEqual(stdout.getvalue(), err + hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_rawpcapclose(self, stdout):
        """Test rawpcapclose help."""
        hlp = ('rawpcapclose\n'
               '  Close the current rawpcap file\n')

        # cannot be done from command, just call it
        self.client.onecmd('help rawpcapclose')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_rawstart(self, stdout):
        """Test rawstart parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: rawstart ARCHI NUM CHANNEL\n'
               '  ARCHI:   m3/a8\n'
               '  NUM:     node num\n'
               '  CHANNEL: sniffer channel\n')

        # Missing port
        self.client.onecmd('rawstart localhost')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Invalid port
        self.client.onecmd('rawstart localhost abc')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Invalid channel
        self.client.onecmd('rawstart m3 123 CHANNEL')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Too many arguments
        self.client.onecmd('rawstart m3 123 11 anotherone')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_stop(self, stdout):
        """Test stop parser errors."""
        hlp = ('Error: Invalid arguments\n'
               'Usage: stop ARCHI NUM\n'
               '  ARCHI: m3/a8\n'
               '  NUM:   node num\n')

        # Missing port
        self.client.onecmd('stop localhost')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Invalid port
        self.client.onecmd('stop localhost abc')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

        # Too many arguments
        self.client.onecmd('stop localhost 123 haha')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)

    def test_stopall(self, stdout):
        """Test stopall help."""
        hlp = 'stopall\n'

        # cannot be done from command, just call it
        self.client.onecmd('help stopall')
        self.assertEqual(stdout.getvalue(), hlp)
        stdout.seek(0)
        stdout.truncate(0)
